/*
 * GUI.java
 *
 * Created on 22 de agosto de 2007, 0:27
 */

package madagascar;

import cube.common.pastry.Ring;
import cube.common.postman.Postman;
import cube.common.services.Impl_service;
import cube.common.updater.PackageData;
import cube.common.updater.Updater;
import cube.common.util.util;
import java.awt.Cursor;
import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.rmi.UnknownHostException;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;
import java.util.logging.Level;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;
import madagascar.files.StateFile;
import rice.environment.Environment;
import rice.p2p.util.XMLObjectInputStream;
import rice.p2p.util.XMLObjectOutputStream;
import rice.pastry.PastryNode;
import service.cubefs.CubeFS_impl;
import service.cubefs.Data_FileState;
import service.cubefs.FileFS;
import service.cubefs.FileManager;
import service.cubefs.MetadataFile;

/**
 *
 * @author  victor Puertas.
 */
public class GUI extends javax.swing.JFrame {
    //Servicio:
    private CubeFS_impl cubeFsInstance = null;
    //Nodo:
    private PastryNode nodo = null;
    //Gestion de conexiones de Cube:
    private Ring anillo = null;
    //Hebra que actualiza la tabla de tareas:
    private MainLoop mainLoop;
    //Host de conexion:
    private String host;
    //Certificado
    private X509Certificate certificado;
    //Puerto del host:
    private int port = 0;
    //Puerto de escucha local:
    private int localPort = 0;
    
    //Formulario que muestra los mensajes del sistema:
    private GUIMessages formMessages = null;
    //Formulario de conexion:
   // private GUIConnection formConnect = new GUIConnection(this,true);
    
    private StateFile archivoEstado;
    
    /**
     * Alias de la conexion al anillo.
     */
    static private String ALIAS_CONNECTION = "madagascar_Ring";
    /**
     * Nombre del anillo en el que se esta conectado.
     */
    private String nombreAnillo="";
    
    //Cadena que muestra el estado a conectado:
    private final String STATE_CONECTION_OK = "<html><body><b>Conectado</b></body></html>";
    //Cadena que muestra el estado a no conectado:
    private final String STATE_CONECTION_NO = "No conectado";
    
    /*TABLA DE TAREAS*/
    static private final String[] header_table ={"Estado", "FileID","Fichero fs", "recibidos/enviados", "Total de fragmentos"};
    
    //Filtro de archivos FS:
    private FileNameExtensionFilter filtro = new FileNameExtensionFilter("Archivos de Cube FS","fs");
    
    //Nombre del archivo de configuracion:
    private String configFile = "madagascar.conf";
    
    /**
     * Contiene informacion de los paquetes empleados por Madagascar.
     * Estea informacion es empleada en las actualizaciones.
     */
    static Vector<PackageData> datos_paquetes = new Vector();
   
    
   /*************************************************************
    * Modelo de la tabla que muestra las tareas
    * Aqui se almacenan los datos necesarios para pedir 
    * informacion al nucleo CubeFS.
    *************************************************************/
    private javax.swing.table.DefaultTableModel tablaInfo = new javax.swing.table.DefaultTableModel(header_table,0) 
                    {
                            Class[] types = new Class [] {
                                java.lang.Object.class, java.lang.String.class, java.lang.String.class, java.lang.Long.class, java.lang.Integer.class
                            };
                            boolean[] canEdit = new boolean [] {
                                false, false, false, false, false
                            };

                            public Class getColumnClass(int columnIndex) {
                                return types [columnIndex];
                            }

                            public boolean isCellEditable(int rowIndex, int columnIndex) {
                                return canEdit [columnIndex];
                            }
                    };
        
    /**
     * Parametros de configuracion de madagascar:
     */
     static Properties madagascarConf = new Properties();
     
    /** Constructor de la clase */
    public GUI() 
    {
        
        initComponents();
        
        /*
         * CARGA DE PARAMETROS DE CONFIGURACION DE MADAGASCAR
         */
        try {
        
            madagascarConf.load(new FileInputStream(Postman.getAppRoot()+File.separator+""+"madagascar"+File.separator+configFile));
        } catch (IOException ex) {
            //Establecemos la configuracion por defecto:
            madagascarConf.setProperty("ipnode","");
            madagascarConf.setProperty("ringname","");
            madagascarConf.setProperty("localPort","8429");
            madagascarConf.setProperty("destinyPort","8429");
        }
        
         /*
         * CARGA DE INFORMACION DE TAREAS
         */
        archivoEstado = new StateFile(Postman.getAppRoot()+File.separator+"madagascar"+File.separator+"madagascarTask.xml");
        archivoEstado.Load();
        
        System.out.println("Versión Java:\t"+System.getProperty("java.version")+" "+System.getProperty("java.vendor"));
        System.out.println("Plataforma:\t"+System.getProperty("os.name")+" "+System.getProperty("os.version"));
    
        String valor = System.getProperty("madagascar.stdout");
        if(valor==null)
        {
            formMessages = new GUIMessages(this,false);
            SbMenssage.setEnabled(true);
        }
        /*
         * CARGAR LA INFORMACION DE LOS PAQUETES EMPLEADOS: IMPORTANTE EL ORDEN EN QUE SE CARGAN.
         */
        PackageData paquete; 
        //El propio madagascar:
        paquete =  util.readPackageInfo(this,"/madagascar/package_info/package_info.xml");
        if(paquete!=null) datos_paquetes.add(paquete);
         
        //CubeFS:
        paquete =  util.readPackageInfo(new CubeFS_impl(),"/service/cubefs/package_info/package_info.xml");
        if(paquete!=null) datos_paquetes.add(paquete);
        
        //Common Component:
        paquete =  util.readPackageInfo(cube.common.postman.Postman.GetRefCryptography(),"/cube/common/package_info/package_info.xml");
        if(paquete!=null) datos_paquetes.add(paquete);
       
          
        /*************************************************************
         * Cargar la informacion de las tareas del archivo de estado:
         *************************************************************/
        Vector<Vector> filas = archivoEstado.getData();
        for(int i=0; i< filas.size();i++)
        {
            tablaInfo.addRow(filas.elementAt(i)); 
        }
        madagascar.util.util.adjustColumWidth(tabla);
        
        /*************************************************************
         * Mostrar informacion de los paquetes instalados:
         *************************************************************/
        Iterator<PackageData> it=datos_paquetes.iterator();
        while(it.hasNext())
        {
            paquete = it.next();
            System.out.println(paquete.public_name+" "+paquete.version_main+"."+paquete.version_second+"."+paquete.version_rev
                        +" "+paquete.codename);
            
        }
    }
     
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        MnuEmergente = new javax.swing.JPopupMenu();
        SbEliminar = new javax.swing.JMenuItem();
        Separador_2 = new javax.swing.JSeparator();
        SbInfoFS = new javax.swing.JMenuItem();
        ContenedorTabla = new javax.swing.JScrollPane();
        tabla = new javax.swing.JTable();
        Herramientas_1 = new javax.swing.JToolBar();
        BtOpenFs = new javax.swing.JButton();
        BtConnect = new javax.swing.JButton();
        BtDisconect = new javax.swing.JButton();
        Herramientas_3 = new javax.swing.JToolBar();
        LbTituloEstado = new javax.swing.JLabel();
        LbEstado = new javax.swing.JLabel();
        Herramientas_2 = new javax.swing.JToolBar();
        BtUpFile = new javax.swing.JButton();
        BtDownFile = new javax.swing.JButton();
        BtPlay = new javax.swing.JButton();
        BtPause = new javax.swing.JButton();
        Herramientas_4 = new javax.swing.JToolBar();
        BtActualizar = new javax.swing.JButton();
        BtActualizar.setCursor(new Cursor(Cursor.HAND_CURSOR));
        CubeFSLogo = new javax.swing.JLabel();
        CubeFSLogo.setCursor(new Cursor(Cursor.HAND_CURSOR));
        InfoPanel = new javax.swing.JTabbedPane();
        dataJob = new javax.swing.JPanel();
        jobInfo = new javax.swing.JLabel();
        dataNode = new javax.swing.JPanel();
        nodeInfo = new javax.swing.JLabel();
        MainMenu = new javax.swing.JMenuBar();
        Archivo = new javax.swing.JMenu();
        SbVerFileFS = new javax.swing.JMenuItem();
        Separador_1 = new javax.swing.JSeparator();
        SbSalir = new javax.swing.JMenuItem();
        Servicio = new javax.swing.JMenu();
        SbConectar = new javax.swing.JMenuItem();
        SbDesconectar = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        SbUpFile = new javax.swing.JMenuItem();
        SbDownFile = new javax.swing.JMenuItem();
        Configuracion = new javax.swing.JMenu();
        SbCert = new javax.swing.JMenuItem();
        SbMenssage = new javax.swing.JMenuItem();
        Ayuda = new javax.swing.JMenu();
        SbAyuda = new javax.swing.JMenuItem();
        SbBug = new javax.swing.JMenuItem();
        SbAbout = new javax.swing.JMenuItem();

        MnuEmergente.setInvoker(tabla);
        SbEliminar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/delete_16x16.png")));
        SbEliminar.setText("Eliminar tarea");
        SbEliminar.setToolTipText("Elimina la tarea pero NO el fichero FS NI EL CONTENIDO DESCARGADO.");
        SbEliminar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbEliminarActionPerformed(evt);
            }
        });

        MnuEmergente.add(SbEliminar);

        MnuEmergente.add(Separador_2);

        SbInfoFS.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/file_binary_16x16.png")));
        SbInfoFS.setText("Informaci\u00f3n del fichero FS");
        SbInfoFS.setToolTipText("Muestra la informaci\u00f3n del fichero FS seleccionado.");
        SbInfoFS.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbInfoFSActionPerformed(evt);
            }
        });

        MnuEmergente.add(SbInfoFS);

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Madagascar front-end");
        setForeground(java.awt.Color.white);
        setIconImage(java.awt.Toolkit.getDefaultToolkit().getImage(java.net.URLClassLoader.getSystemResource("madagascar/recursos/iconApp_16x16.png")));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        ContenedorTabla.setBackground(new java.awt.Color(255, 255, 255));
        ContenedorTabla.setDoubleBuffered(true);
        tabla.setModel(tablaInfo);
        tabla.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                tablaMouseClicked(evt);
                jTable1MouseClicked(evt);
            }
        });

        ContenedorTabla.setViewportView(tabla);

        Herramientas_1.setFloatable(false);
        Herramientas_1.setBorderPainted(false);
        Herramientas_1.setDoubleBuffered(true);
        BtOpenFs.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/file_binary_32x32.png")));
        BtOpenFs.setToolTipText("Consulta el contenido de un fichero fs.");
        BtOpenFs.setDoubleBuffered(true);
        BtOpenFs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BtOpenFsActionPerformed(evt);
            }
        });

        Herramientas_1.add(BtOpenFs);

        BtConnect.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/connect_32x32.png")));
        BtConnect.setToolTipText("Inicial la conexi\u00f3n con un anillo.");
        BtConnect.setDoubleBuffered(true);
        BtConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BtConnectActionPerformed(evt);
            }
        });

        Herramientas_1.add(BtConnect);

        BtDisconect.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/disconnect_32x32.png")));
        BtDisconect.setToolTipText("Cierra la conexi\u00f3n actual con el anillo.");
        BtDisconect.setDoubleBuffered(true);
        BtDisconect.setEnabled(false);
        BtDisconect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BtDisconectActionPerformed(evt);
            }
        });

        Herramientas_1.add(BtDisconect);

        Herramientas_3.setBorder(null);
        Herramientas_3.setFloatable(false);
        LbTituloEstado.setText("    Estado:  ");
        Herramientas_3.add(LbTituloEstado);

        LbEstado.setText("No conectado.");
        LbEstado.setToolTipText("Indica si el servicio est\u00e1 conectado a un anillo.");
        Herramientas_3.add(LbEstado);

        Herramientas_2.setBorder(null);
        Herramientas_2.setFloatable(false);
        Herramientas_2.setDoubleBuffered(true);
        BtUpFile.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/up_32x32.png")));
        BtUpFile.setToolTipText("Subir archivo.");
        BtUpFile.setDoubleBuffered(true);
        BtUpFile.setEnabled(false);
        BtUpFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BtUpFileActionPerformed(evt);
            }
        });

        Herramientas_2.add(BtUpFile);

        BtDownFile.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/down_32x32.png")));
        BtDownFile.setToolTipText("Bajar archivo.");
        BtDownFile.setDoubleBuffered(true);
        BtDownFile.setEnabled(false);
        BtDownFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BtDownFileActionPerformed(evt);
            }
        });

        Herramientas_2.add(BtDownFile);

        BtPlay.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/play_32x32.png")));
        BtPlay.setToolTipText("Continua la descarga/subida de un archivo");
        BtPlay.setEnabled(false);
        BtPlay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BtPlayActionPerformed(evt);
            }
        });

        Herramientas_2.add(BtPlay);

        BtPause.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/stop_32x32.png")));
        BtPause.setToolTipText("Detiene la subida/descarga de un archivo");
        BtPause.setEnabled(false);
        BtPause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BtPauseActionPerformed(evt);
            }
        });

        Herramientas_2.add(BtPause);

        Herramientas_4.setBorder(null);
        BtActualizar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/actualizar_32x32.png")));
        BtActualizar.setToolTipText("Buscar actualizaciones");
        BtActualizar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BtActualizarActionPerformed(evt);
            }
        });

        Herramientas_4.add(BtActualizar);

        CubeFSLogo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/CubeFS.png")));
        CubeFSLogo.setToolTipText("Visita la web del proyecto.");
        CubeFSLogo.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        CubeFSLogo.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                CubeFSLogoMouseClicked(evt);
            }
        });

        InfoPanel.setDoubleBuffered(true);
        jobInfo.setBackground(new java.awt.Color(204, 204, 204));
        jobInfo.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jobInfo.setDoubleBuffered(true);

        org.jdesktop.layout.GroupLayout dataJobLayout = new org.jdesktop.layout.GroupLayout(dataJob);
        dataJob.setLayout(dataJobLayout);
        dataJobLayout.setHorizontalGroup(
            dataJobLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(dataJobLayout.createSequentialGroup()
                .addContainerGap()
                .add(jobInfo, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 939, Short.MAX_VALUE)
                .addContainerGap())
        );
        dataJobLayout.setVerticalGroup(
            dataJobLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(dataJobLayout.createSequentialGroup()
                .addContainerGap()
                .add(jobInfo, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 97, Short.MAX_VALUE)
                .addContainerGap())
        );
        InfoPanel.addTab("resumen", dataJob);

        dataNode.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                dataNodeComponentShown(evt);
            }
        });

        nodeInfo.setBackground(new java.awt.Color(204, 204, 204));
        nodeInfo.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        nodeInfo.setDoubleBuffered(true);

        org.jdesktop.layout.GroupLayout dataNodeLayout = new org.jdesktop.layout.GroupLayout(dataNode);
        dataNode.setLayout(dataNodeLayout);
        dataNodeLayout.setHorizontalGroup(
            dataNodeLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(dataNodeLayout.createSequentialGroup()
                .addContainerGap()
                .add(nodeInfo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 939, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        dataNodeLayout.setVerticalGroup(
            dataNodeLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, dataNodeLayout.createSequentialGroup()
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(nodeInfo, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 97, Short.MAX_VALUE)
                .addContainerGap())
        );
        InfoPanel.addTab("nodo", dataNode);

        Archivo.setText("Archivo");
        SbVerFileFS.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/file_binary_16x16.png")));
        SbVerFileFS.setText("Ver archivo fs");
        SbVerFileFS.setToolTipText("Permite consultar el contenido de un fichero fs.");
        SbVerFileFS.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbVerFileFSActionPerformed(evt);
            }
        });

        Archivo.add(SbVerFileFS);

        Archivo.add(Separador_1);

        SbSalir.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/exit_16x16.png")));
        SbSalir.setText("Salir");
        SbSalir.setToolTipText("Salir de la aplicaci\u00f3n.");
        SbSalir.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbSalirActionPerformed(evt);
            }
        });

        Archivo.add(SbSalir);

        MainMenu.add(Archivo);

        Servicio.setText("Servicio");
        SbConectar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/connect_16x16.png")));
        SbConectar.setText("Conectar");
        SbConectar.setToolTipText("Conecta con el anillo.");
        SbConectar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbConectarActionPerformed(evt);
            }
        });

        Servicio.add(SbConectar);

        SbDesconectar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/disconnect_16x16.png")));
        SbDesconectar.setText("Desconectar");
        SbDesconectar.setToolTipText("Desconecta del anillo.");
        SbDesconectar.setEnabled(false);
        SbDesconectar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbDesconectarActionPerformed(evt);
            }
        });

        Servicio.add(SbDesconectar);

        Servicio.add(jSeparator2);

        SbUpFile.setText("Subir archivo al anillo.");
        SbUpFile.setToolTipText("Permite subir un archivo y crear el fichero fs.");
        SbUpFile.setEnabled(false);
        SbUpFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbUpFileActionPerformed(evt);
            }
        });

        Servicio.add(SbUpFile);

        SbDownFile.setText("Bajar archivo");
        SbDownFile.setToolTipText("Extrae un fichero del anillo.");
        SbDownFile.setEnabled(false);
        SbDownFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbDownFileActionPerformed(evt);
            }
        });

        Servicio.add(SbDownFile);

        MainMenu.add(Servicio);

        Configuracion.setText("Configuraci\u00f3n");
        SbCert.setText("Administrador de certificados");
        SbCert.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbCertActionPerformed(evt);
            }
        });

        Configuracion.add(SbCert);

        SbMenssage.setText("Mensajes del sistema");
        SbMenssage.setEnabled(false);
        SbMenssage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbMenssageActionPerformed(evt);
            }
        });

        Configuracion.add(SbMenssage);

        MainMenu.add(Configuracion);

        Ayuda.setText("Ayuda");
        SbAyuda.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/help_16x16.png")));
        SbAyuda.setText("Ayuda");
        SbAyuda.setEnabled(false);
        SbAyuda.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbAyudaActionPerformed(evt);
            }
        });

        Ayuda.add(SbAyuda);

        SbBug.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/error_16x16.png")));
        SbBug.setText("Informar de error");
        SbBug.setEnabled(false);
        SbBug.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbBugActionPerformed(evt);
            }
        });

        Ayuda.add(SbBug);

        SbAbout.setIcon(new javax.swing.ImageIcon(getClass().getResource("/madagascar/recursos/iconApp_16x16.png")));
        SbAbout.setText("Acerca de...");
        SbAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SbAboutActionPerformed(evt);
            }
        });

        Ayuda.add(SbAbout);

        MainMenu.add(Ayuda);

        setJMenuBar(MainMenu);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(Herramientas_1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(Herramientas_2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 198, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 510, Short.MAX_VALUE)
                .add(CubeFSLogo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 130, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .add(Herramientas_3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 925, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(Herramientas_4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 61, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(InfoPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 968, Short.MAX_VALUE)
                .addContainerGap())
            .add(ContenedorTabla, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 992, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .addContainerGap()
                        .add(CubeFSLogo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 38, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                        .add(org.jdesktop.layout.GroupLayout.LEADING, Herramientas_2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .add(org.jdesktop.layout.GroupLayout.LEADING, Herramientas_1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 47, Short.MAX_VALUE)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(ContenedorTabla, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 366, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(InfoPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 148, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(Herramientas_4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 56, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(Herramientas_3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 56, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
        );
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-1002)/2, (screenSize.height-687)/2, 1002, 687);
    }// </editor-fold>//GEN-END:initComponents

    private void SbDownFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbDownFileActionPerformed
        downFile();
    }//GEN-LAST:event_SbDownFileActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        System.out.println("cerrando madagascar...") ;
        exitGUI();
    }//GEN-LAST:event_formWindowClosing

    private void SbUpFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbUpFileActionPerformed
        upFile();
    }//GEN-LAST:event_SbUpFileActionPerformed

    private void dataNodeComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_dataNodeComponentShown
        refreshInfoNodo();
    }//GEN-LAST:event_dataNodeComponentShown

    private void CubeFSLogoMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_CubeFSLogoMouseClicked
        cube.common.util.Browser.displayURL("http://cube-grid.sourceforge.net");
    }//GEN-LAST:event_CubeFSLogoMouseClicked

    private void SbMenssageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbMenssageActionPerformed
            formMessages.setVisible(true);
    }//GEN-LAST:event_SbMenssageActionPerformed

    private void SbAboutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbAboutActionPerformed
        GUIAbout form = new GUIAbout(this,true);
        form.setVersion(datos_paquetes.elementAt(0).version_main+"."+datos_paquetes.elementAt(0).version_second+"."+datos_paquetes.elementAt(0).version_rev
                        +" "+datos_paquetes.elementAt(0).codename);
        form.setVisible(true);
    }//GEN-LAST:event_SbAboutActionPerformed

    private void SbVerFileFSActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbVerFileFSActionPerformed
        getInfoFS();
    }//GEN-LAST:event_SbVerFileFSActionPerformed

    private void SbCertActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbCertActionPerformed
        GUIaddCertified form = new GUIaddCertified(this,false);
        form.setVisible(true);
    }//GEN-LAST:event_SbCertActionPerformed

    private void BtActualizarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_BtActualizarActionPerformed

        findUpdate();
    }//GEN-LAST:event_BtActualizarActionPerformed
    
   /*************************************************************
    * Menu contextual: eliminar tarea.
    *************************************************************/
    private void SbEliminarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbEliminarActionPerformed
        int row = tabla.getSelectedRow();
        
        if(cubeFsInstance==null) 
        {
            JOptionPane.showMessageDialog(this,"No se puede eliminar la tarea. Debe estar iniciado CubeFS.","Error",JOptionPane.ERROR_MESSAGE);
            return;
        }
        if(row > -1)
        {
             String fileID = (String) tablaInfo.getValueAt(row,2);
             cubeFsInstance.cleanInfoState(fileID);
             tablaInfo.removeRow(row) ;
             archivoEstado.save(tablaInfo.getDataVector());
        }
    }//GEN-LAST:event_SbEliminarActionPerformed

   /*************************************************************
    * Menu contextual: informacion fichero FS.
    *************************************************************/
    private void SbInfoFSActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbInfoFSActionPerformed
        int row = tabla.getSelectedRow();
        if(row > -1)
        {
             infoFS((String) tablaInfo.getValueAt(row,2));   
        }
    }//GEN-LAST:event_SbInfoFSActionPerformed

    private void SbDesconectarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbDesconectarActionPerformed
        disconnectRing();
    }//GEN-LAST:event_SbDesconectarActionPerformed

    private void SbConectarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbConectarActionPerformed
        connectRing();
    }//GEN-LAST:event_SbConectarActionPerformed

    private void BtDisconectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_BtDisconectActionPerformed
        disconnectRing();
    }//GEN-LAST:event_BtDisconectActionPerformed

    private void SbBugActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbBugActionPerformed
        GUIBug form = new GUIBug(this,true);
        form.setVisible(true);
    }//GEN-LAST:event_SbBugActionPerformed

    private void jTable1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jTable1MouseClicked
// TODO add your handling code here:
    }//GEN-LAST:event_jTable1MouseClicked

    private void BtPlayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_BtPlayActionPerformed
        int row = tabla.getSelectedRow();
        String fileID = null;
        if(row > -1)
        {
            fileID = (String) tablaInfo.getValueAt(row,1);   
        }
        if(cubeFsInstance.infoState(fileID)==255 || cubeFsInstance.infoState(fileID)!= FileManager.STATE_PAUSE)
            cubeFsInstance.initializeOne(fileID);
        else
            cubeFsInstance.continueOne(fileID);
    }//GEN-LAST:event_BtPlayActionPerformed

    private void BtPauseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_BtPauseActionPerformed
        int row = tabla.getSelectedRow();
        String fileID = null;
        if(row > -1)
        {
            fileID = (String) tablaInfo.getValueAt(row,1);   
        }
        cubeFsInstance.pauseOne(fileID);
        
    }//GEN-LAST:event_BtPauseActionPerformed

    private void tablaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tablaMouseClicked

        //Si click con el boton derecho: mostrar el menu contextual.
        if(evt.getButton()==evt.BUTTON3)
            MnuEmergente.show(evt.getComponent(), evt.getX(), evt.getY());        
    }//GEN-LAST:event_tablaMouseClicked

    private void SbSalirActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbSalirActionPerformed

       exitGUI();
    }//GEN-LAST:event_SbSalirActionPerformed

    private void SbAyudaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SbAyudaActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_SbAyudaActionPerformed

    private void BtDownFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_BtDownFileActionPerformed
        downFile();
    }//GEN-LAST:event_BtDownFileActionPerformed

    private void BtConnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_BtConnectActionPerformed
       connectRing();     
    }//GEN-LAST:event_BtConnectActionPerformed

    private void BtUpFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_BtUpFileActionPerformed
            upFile();
    }//GEN-LAST:event_BtUpFileActionPerformed

    private void BtOpenFsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_BtOpenFsActionPerformed
        getInfoFS();
    }//GEN-LAST:event_BtOpenFsActionPerformed
    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenu Archivo;
    private javax.swing.JMenu Ayuda;
    private javax.swing.JButton BtActualizar;
    private javax.swing.JButton BtConnect;
    private javax.swing.JButton BtDisconect;
    private javax.swing.JButton BtDownFile;
    private javax.swing.JButton BtOpenFs;
    private javax.swing.JButton BtPause;
    private javax.swing.JButton BtPlay;
    private javax.swing.JButton BtUpFile;
    private javax.swing.JMenu Configuracion;
    private javax.swing.JScrollPane ContenedorTabla;
    private javax.swing.JLabel CubeFSLogo;
    private javax.swing.JToolBar Herramientas_1;
    private javax.swing.JToolBar Herramientas_2;
    private javax.swing.JToolBar Herramientas_3;
    private javax.swing.JToolBar Herramientas_4;
    private javax.swing.JTabbedPane InfoPanel;
    private javax.swing.JLabel LbEstado;
    private javax.swing.JLabel LbTituloEstado;
    private javax.swing.JMenuBar MainMenu;
    private javax.swing.JPopupMenu MnuEmergente;
    private javax.swing.JMenuItem SbAbout;
    private javax.swing.JMenuItem SbAyuda;
    private javax.swing.JMenuItem SbBug;
    private javax.swing.JMenuItem SbCert;
    private javax.swing.JMenuItem SbConectar;
    private javax.swing.JMenuItem SbDesconectar;
    private javax.swing.JMenuItem SbDownFile;
    private javax.swing.JMenuItem SbEliminar;
    private javax.swing.JMenuItem SbInfoFS;
    private javax.swing.JMenuItem SbMenssage;
    private javax.swing.JMenuItem SbSalir;
    private javax.swing.JMenuItem SbUpFile;
    private javax.swing.JMenuItem SbVerFileFS;
    private javax.swing.JSeparator Separador_1;
    private javax.swing.JSeparator Separador_2;
    private javax.swing.JMenu Servicio;
    private javax.swing.JPanel dataJob;
    private javax.swing.JPanel dataNode;
    private javax.swing.JSeparator jSeparator2;
    javax.swing.JLabel jobInfo;
    javax.swing.JLabel nodeInfo;
    private javax.swing.JTable tabla;
    // End of variables declaration//GEN-END:variables
 
    /**
     * Devuelve el fileID del trabajo actualmente seleccionado.
     * @return null si no hay seleccionado ningun trabajo.
     */
    public String nowFileID() 
    {
        int row = tabla.getSelectedRow();
        String fileID = null;
        if(row > -1)
        {
             return (String) tablaInfo.getValueAt(row,1);   
        }
        return null;
    }
    /**
     * Carga el formulario de actualizaciones.
     */
    private void findUpdate()
    {
        GUIUpdate form = new GUIUpdate(this,false);
        form.setDataService(datos_paquetes);
        form.setVisible(true);
    }
    
    /**
     * Permite consultar los datos de un fichero FS. 
     */
    private void getInfoFS()
    {
        int retorno;
        String datos = null;
        FileFS archivo;
        JFileChooser filechooser = new JFileChooser();
        filechooser.setDialogTitle("Abrir fichero fs:");
        filechooser.setFileFilter(filtro);
        retorno = filechooser.showOpenDialog(null);
        
        if (retorno == JFileChooser.APPROVE_OPTION)
        {
            try {
                infoFS(filechooser.getSelectedFile().getCanonicalPath());
            } catch (IOException ex) {}
        }
        else return;
    }
    /**
     * Muestra un formulario con la informacion del fichero FS
     * pasado como argumento.
     */
    private void infoFS(String fileFS)
    {
        FileFS archivo;
        try {
                archivo = new FileFS(fileFS);
                archivo.Open();
            } catch (IOException ex) {
                JOptionPane.showMessageDialog(this,"No se encuentra el fichero FS.","Error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            
            GUIFsInfo form = new GUIFsInfo(this,true);
            MetadataFile metadatos = archivo.Get_info_metadata();
            String descripcion = madagascar.util.util.HTML_BODY+metadatos.GetDescription().replaceAll("\n","<br>")+
                                 madagascar.util.util.HTML_END;
            
            form.setData(descripcion, metadatos.GetLicense(), archivo.toHtml());
            archivo.Close();
            form.setVisible(true);
    }
    /**
     * Finaliza la sesion Cube FS.
     */
    private void exitCubeFS()
    {
        if(cubeFsInstance!=null)
        cubeFsInstance.ServiceStop(cube.common.services.Impl_service.TERMINATE);
        
        if(anillo!=null && anillo.closeNode(ALIAS_CONNECTION))
        {
            mainLoop.pauseLoop();
            LbEstado.setText(STATE_CONECTION_NO);
        }
        else
            LbEstado.setText(STATE_CONECTION_OK);
        System.gc();
    }
    
    /**
     * Cierra Madagascar.
     */
    private void exitGUI()
    {
        //exitCubeFS();
        disconnectRing();
        //Salvar la informacion de las tareas:
        archivoEstado.save(tablaInfo.getDataVector());
        try {
            //Salvar los cambios en la configuracion:
            madagascarConf.store(new FileOutputStream(Postman.getAppRoot()+File.separator+""+"madagascar"+File.separator+configFile),"Archivo de configuracion de madagascar.");
        } catch (IOException ex) {}
        System.exit(0);
    }
    
    /**
     * Actualiza la informacion del nodo de la pestaña: nodo.
     */
    private void refreshInfoNodo()
    {
        String nodoID = "No disponible.";
        String nombreAnillo = nodoID;
        String espacioDisco = nodoID;
        String espacioCache = nodoID;
        
        if(cubeFsInstance!=null)
        {
            nodoID = nodo.getId().toStringFull();
            nombreAnillo = anillo.getRingName(this.ALIAS_CONNECTION);
            espacioDisco = cube.common.util.util.parserBytes2Size(cubeFsInstance.getHdSpace());
            espacioCache = cube.common.util.util.parserBytes2Size(cubeFsInstance.getCacheSpace());
        }
        
        nodeInfo.setText(madagascar.util.util.HTML_BODY+
                "<b>Nombre del nodo:</b> "+Postman.getNodeName()+"<br>"+
                "<b>Nodo id: </b>"+nodoID+"<br>"+
                "<b>Nombre del anillo: </b>"+nombreAnillo+"<br><br>"+
                "<b>Espácio cedido en disco: </b>"+espacioDisco+"<br>"+
                "<b>Espácio para cache: </b>"+espacioCache+"<br>"
                +madagascar.util.util.HTML_END);
    }
    
    /**
     * Conecta con un anillo.
     */
    private void connectRing()
    {
            GUIConnection formConnect = new GUIConnection(this,true);
            formConnect.setVisible(true);
            
            if(formConnect.getAction() == 0) return;
            
            host = formConnect.getHost();
            
            if(formConnect.useCert())
                certificado = formConnect.getCert();
            
            //Puertos usados:
            port = formConnect.getPort();
            localPort = formConnect.getLocalPort();  

            this.setCursor(new Cursor(Cursor.WAIT_CURSOR));
            
            anillo = new Ring();
            
            //Propiedades de la conexion:
            Properties conf = new Properties();
            conf.setProperty("ring.alias", ALIAS_CONNECTION);
            
            conf.setProperty("ring.type", "PUBLIC_RING");
            if(formConnect.createRing())
            {
                conf.setProperty("ring.create","");
                conf.setProperty("ring.name", formConnect.getNameRing());
            }
                 
            try 
            {
                if(formConnect.useCert())
                {
                    nodo = anillo.JoinToRing(conf,certificado, localPort,new InetSocketAddress(InetAddress.getByName(host), port),
                            //cargamos le pasamos a pastry ring.params
                            new Environment(Postman.getCubeDirectoryRoot()+File.separator+"conf"+File.separator+"ring"));
                  
                    try 
                    {
                        cubeFsInstance = new CubeFS_impl(Ring.PUBLIC_RING, nodo, certificado); 
                    } 
                    catch (IOException ex) 
                    {
                         this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                        JOptionPane.showMessageDialog(this,"Ha ocurrido un problema al iniciar Cube File Service.","Error",JOptionPane.ERROR_MESSAGE);
                        LbEstado.setText(STATE_CONECTION_NO);
                        return;
                    }
                }
                else
                {
                    nodo = anillo.JoinToRing(conf, localPort, new InetSocketAddress(InetAddress.getByName(host), port),
                            new Environment(Postman.getCubeDirectoryRoot()+File.separator+"conf"+File.separator+"ring"));
                  
                    try 
                    {
                        cubeFsInstance = new CubeFS_impl(Ring.PUBLIC_RING, nodo, null); 
                    } 
                    catch (IOException ex) 
                    {
                        this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                        JOptionPane.showMessageDialog(this,"Ha ocurrido un problema al iniciar Cube File Service.","Error",JOptionPane.ERROR_MESSAGE);
                        LbEstado.setText(STATE_CONECTION_NO);
                        
                        return;
                    }
                }
                cubeFsInstance.ServiceStart(null);
                LbEstado.setText(STATE_CONECTION_OK);
                
                mainLoop = new MainLoop(cubeFsInstance, tablaInfo, this);
                mainLoop.start();
                refreshInfoNodo();
                
                BtConnect.setEnabled(false);
                BtUpFile.setEnabled(true);
                BtDownFile.setEnabled(true);
                BtDisconect.setEnabled(true);
                BtPause.setEnabled(true);
                BtPlay.setEnabled(true);

                
                SbDownFile.setEnabled(true);
                SbUpFile.setEnabled(true);
                SbConectar.setEnabled(false);
                SbDesconectar.setEnabled(true);
                nombreAnillo = anillo.getRingName(ALIAS_CONNECTION);
                
                this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                return;
                
            } catch (UnknownHostException ex) {
                JOptionPane.showMessageDialog(this, "No se encuentra el host de conexion: "+host,"Error",JOptionPane.ERROR_MESSAGE);
                //System.err.println("Error: No se encuentra el host de conexion: "+host);
                
            } catch (IOException ex) {
                JOptionPane.showMessageDialog(this, "No se puede conectar con el anillo.","Error",JOptionPane.ERROR_MESSAGE); 
            } catch(Exception ex){System.out.println("Fallo general al conectar con el anillo. "+ex.toString());}
            
            LbEstado.setText(STATE_CONECTION_NO);      
    }
    
    /**
     * Desconecta de un anillo.
     */
    private void disconnectRing()
    {
        this.setCursor(new Cursor(Cursor.WAIT_CURSOR));
        exitCubeFS();
        //Botones:
        BtConnect.setEnabled(true);
        BtUpFile.setEnabled(false);
        BtDownFile.setEnabled(false);
        BtDisconect.setEnabled(false);
        BtPause.setEnabled(false);
        BtPlay.setEnabled(false);
        
        //Menus:
        SbDownFile.setEnabled(false);
        SbUpFile.setEnabled(false);
        SbConectar.setEnabled(true);
        SbDesconectar.setEnabled(false);

        
        if(mainLoop!=null) mainLoop.pauseLoop();
        this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
    
    private void upFile()
    {
         GUIUpFile form = new GUIUpFile(this,true);
        form.setVisible(true);
        int retorno;
        String archivoFs = null;
        String archivoTrabajo = null;
       
        if (form.getOption() == 1)
        {
            archivoFs = form.getFsFile()+".fs";
            archivoTrabajo = form.getFile();
        
       
        
            GUIMetaData myMetadataForm = new GUIMetaData(this,true);
            myMetadataForm.setVisible(true);

            String fileID = cubeFsInstance.up_file(archivoTrabajo,archivoFs, new MetadataFile(myMetadataForm.getDescripcion(), myMetadataForm.getAutor(), 
                                        new Date(),myMetadataForm.getCopyright(),myMetadataForm.getLicencia()),form.getPersistent());

            //Vemos si ya existe la columna con el mismo fileID en cuyo caso no se añade:
            for(int i=0; i<tablaInfo.getRowCount(); i++)
                if(((Vector)tablaInfo.getDataVector().elementAt(i)).elementAt(1).equals(fileID))  return;
            
            String fileFS = cubeFsInstance.infoJob(fileID).fsFile;
            //Object [] pre_Fila = new Object [] {"",fileID,fileFS, new Long(0), null};
            //estado.add(fileID);
            Object [] Fila = new Object [] {"Subiendo",fileID,fileFS, new Long(0), 0};
            tablaInfo.addRow(Fila);
        }
    }
    
    private void downFile()
    {
        String archivoFs = null;
        String archivoDestino = null;
        int retorno;
        
        JFileChooser filechooser = new JFileChooser();
        filechooser.setDialogTitle("Abrir fichero fs:");
        filechooser.setFileFilter(filtro);
        retorno = filechooser.showOpenDialog(null);
        
        if (retorno == JFileChooser.APPROVE_OPTION)
        {
            try {
                archivoFs = filechooser.getSelectedFile().getCanonicalPath();
            } catch (IOException ex) {
                return;
            }
        }
        else return;
        
        filechooser = new JFileChooser();
        filechooser.setDialogTitle("Guardar en ...");
        retorno = filechooser.showSaveDialog(null);
        
        if (retorno == JFileChooser.APPROVE_OPTION)
        {
            try {
                archivoDestino = filechooser.getSelectedFile().getCanonicalPath();
            } catch (IOException ex) {
                return;
            }
        }
        else return;
        
        String fileID = cubeFsInstance.down_file(archivoDestino,archivoFs);
        
        //Vemos si ya existe la columna con el mismo fileID en cuyo caso no se añade:
        for(int i=0; i<tablaInfo.getRowCount(); i++)
            if(((Vector)tablaInfo.getDataVector().elementAt(i)).elementAt(1).equals(fileID))  return;
            
        Data_FileState datos = cubeFsInstance.infoJob(fileID);    
        Object [] Fila = new Object [] {"Descargando",fileID,datos.fsFile, new Long(0), datos.totalTrack};
        tablaInfo.addRow(Fila);
    }
}
