/*
* Title			:   MainLoop.java
* Package		:   madagascar
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Hilo con las tareas periodicas como la mantenciaon de la informacion
*                           de la tabla con las tareas.  
*                           		
* Created on            :   02/10/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package madagascar;

import java.util.Vector;
import service.cubefs.CubeFS_impl;
import service.cubefs.Data_FileState;
import service.cubefs.FileManager;

/**
 * Hebra que se encarga de actualizar la tabla con la informacion de las tareas
 * creadas por Madagascar. Simplemente va realizando peticiones de informacion sobre
 * los trabajos a CubeFS.
 * @author victor Puertas.
 */
public class MainLoop extends Thread {
    //Estado del mainloop:
    private byte estado_bucle = 0;
    //Referecia a la instancia CubeFS:
    private CubeFS_impl instanciaCubeFS = null;
    //Tabla en la que actualizar los datos:
    private javax.swing.table.DefaultTableModel tablaInfo = null;
    //Referencia al GUI:
    private GUI myGUI;
    
    /** 
     * Crea una nueva instancia del mainloop.
     * @param Modelo de datos asociado a la tabla del GUI.
     */
    public MainLoop(CubeFS_impl instanciaCubeFS, javax.swing.table.DefaultTableModel tablaInfo, GUI theGUI) {
        this.tablaInfo = tablaInfo;
        this.instanciaCubeFS = instanciaCubeFS;
        myGUI = theGUI;
        
        //Nombre del thread:
        setName("madagascar-mainloop");
    }
    
    /**
     * Pausa el hilo que actualiza la tabla con los datos de la tareas.
     */
    public synchronized void pauseLoop() { estado_bucle = 1; }
    /**
     * Finaliza el hilo que actualiza la tabla con los datos de la tareas.
     */
    public synchronized void TerminateLoop() { estado_bucle = 2;}
    /**
     * Reanuda el hilo que actualiza la tabla con los datos de la tareas.
     */
    public synchronized void ContinueLoop() { estado_bucle = 0;  notifyAll(); }
    
    /**
     * Cuerpo del Hilo.
     */
    public void run()
    {
        Vector<Object []> vectorDatos;
        byte estadoFM, tipo;
        String texto_estado="";
        String fileID;
        while(estado_bucle!=2)
        {
         
            for(int i=0; i<tablaInfo.getRowCount(); i++)
            {
                fileID = (String) tablaInfo.getValueAt(i,1);
                
                Data_FileState datosTarea = instanciaCubeFS.infoJob(fileID);
                if(datosTarea != null)
                {
                    //track ok:
                    tablaInfo.setValueAt(datosTarea.totalTrack,i,4); 
                    tablaInfo.setValueAt(datosTarea.trackok,i,3); 
                }
                
                //Obtenemos el estado del FileManager:
                estadoFM = instanciaCubeFS.infoState(fileID);
                
                if(estadoFM==FileManager.STATE_PAUSE)
                {
                    tablaInfo.setValueAt("Detenido",i,0);
                }
                else
                {
                    if(estadoFM==(byte)255 || datosTarea.getState() == FileManager.STATE_TERMINATE)
                        tablaInfo.setValueAt("Finalizado",i,0);
                    else
                    {
                        if(datosTarea.getState()==FileManager.STATE_DOWN)
                            tablaInfo.setValueAt("Descargando",i,0);
                        else
                            tablaInfo.setValueAt("Subiendo",i,0);
                    }
                }
                
            }
            
            //Añadir informacion de resumen:
            fileID = myGUI.nowFileID();
            if(fileID!=null)
            {
                Data_FileState job = instanciaCubeFS.infoJob(fileID);
                if(job!=null)
                {
                     String tipoTrabajo = null, size="";
                     switch(job.getState())
                     {
                         case FileManager.STATE_UP:
                             tipoTrabajo = "Subido";
                             break;
                         case FileManager.STATE_DOWN:
                             tipoTrabajo = "Descargado";
                             break;
                         case FileManager.STATE_TERMINATE:
                             tipoTrabajo = "Terminado";
                             break;
                     }
                     
                     size = "/"+cube.common.util.util.parserBytes2Size(job.track_long*job.totalTrack);
                     
                     myGUI.jobInfo.setText(madagascar.util.util.HTML_BODY+"<br><b>Archivo FS:</b> "+job.fsFile+"<br>"+
                             "<b>"+tipoTrabajo+":</b> "+cube.common.util.util.parserBytes2Size(job.track_long*job.trackok)+size+"<br>"+
                             "<b>Tracks erroneos:</b> "+job.trackbad+"<br>"+
                             "<b>Archivo de trabajo:</b> "+job.workfile +
                             madagascar.util.util.HTML_END);
                }
                else{ myGUI.jobInfo.setText("");}
            }
            
            //Vemos si hay alguna orden:
            switch(estado_bucle)
            {
                case 0: try { this.sleep(1000); } catch (InterruptedException ex) {} break;
                case 1:pauseLoop();     
            }
        }
    }
}
