/*
* Title			:   StateFile.java
* Package		:   madagascar.files
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Permite manegar el archivo de estado de tareas de Madagascar.
*                           		
* Created on            :   16/10/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*/

package madagascar.files;


import java.io.*;
import java.util.Vector;
import org.xmlpull.v1.*;

/**
 * Formato del archivo: XML.
 * tipo: <fileID><file FS><fragmentos operados><total de fragmentos>
 * @author victor Puertas.
 *@todo Comprobar la version del archivo XML. Si es menor a la esperada no leerlo.
 */
public class StateFile {
    
    //Archivo de estado.
    private String file;
    //Vector de vectores. Cada uno de estos ultimos representa una fila.
    private Vector filas  = new Vector();
    //Vector que almacena una fila:
    private Vector fila;
    
    /**
     * Numero de campos a almacenar por cada trabajo.
     */
    static private final byte NUMBER_DATA = 5;
    /**
     * Version del formato de archivo.
     */ 
    static public final String VER_FILE = "1.0";
    /**
     * Comentarios previos al tag de inicio XML:
     */
    static private final String COMMENT = "<!-- Archivo de estado de tareas de Madagascar -->";
    
    /** Cronstructor de la clase. */
    public StateFile(String file) { this.file = file;}
    
    public Vector<Vector> getData() { return filas; }
    
    /**
     * Abre un fichero de estado y extrae la informacion.
     * El fichero posee formato XML.
     */
    public void Load()
    {
        FileInputStream fin = null;
        try { fin = new FileInputStream(file); } catch (FileNotFoundException ex) { return; }
       
        try
        {
            XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
            factory.setNamespaceAware(true);
            XmlPullParser xpp = factory.newPullParser();
            xpp.setInput(fin,null);
            
            //Almacena el ultimo tag procesado:
            String lastTag = "";
            
            //filas = new Vector();
            
            //Gestion de los eventos
            int eventType = xpp.getEventType();
            while (eventType != XmlPullParser.END_DOCUMENT) 
             {
                switch(eventType)
                {
                    case XmlPullParser.START_TAG:
                        lastTag = xpp.getName();
                        
                        if(xpp.getName()=="job")
                        {
                            fila = new Vector();
                            fila.add(0,"");
                        }
                        break;
                        
                    case XmlPullParser.END_TAG:
                        lastTag = "";
                        
                        if(xpp.getName()=="job")
                        {
                            //Compribamos el tamaño de la fila para ver si hay algun error:
                            if(fila.size()==NUMBER_DATA)
                                filas.add(fila);
                        }
                        break;
                        
                    case XmlPullParser.TEXT:
                        if(lastTag=="fileID")
                            fila.add(1,xpp.getText());
                        
                        if(lastTag=="fileFS")
                            fila.add(2,xpp.getText());
                        
                        if(lastTag=="track")
                            try{ fila.add(3,Integer.parseInt(xpp.getText())); }
                            catch (NumberFormatException err){fila.add(3,0);}
                        
                        if(lastTag=="totalTrack")
                            try{ fila.add(4,Integer.parseInt(xpp.getText())); }
                            catch (NumberFormatException err){fila.add(4,0);}    
                        break;
                }
                eventType = xpp.next();
            }
         
        }
        catch (XmlPullParserException ex) { return; }
        catch (IOException ex) { return; }
    }
    
    public void save(Vector<Vector> datos)
    {
        PrintStream fout;
        File auxfile = new File (file);
        auxfile.delete();
        
        try 
        {
            auxfile.createNewFile();
            fout = new PrintStream(auxfile);
            
            //Comentarios previos:
            fout.println(this.COMMENT);
            
            //Version del formato de archivo:
            fout.println("<Madagascar version=\""+this.VER_FILE+"\">");
            
            for(int i=0; i<datos.size(); i++)
            {
                fout.println("<job>");
                fout.println("<fileID>"+datos.elementAt(i).elementAt(1)+"</fileID>");
                fout.println("<fileFS>"+datos.elementAt(i).elementAt(2)+"</fileFS>");
                fout.println("<track>"+datos.elementAt(i).elementAt(3)+"</track>");
                fout.println("<totalTrack>"+datos.elementAt(i).elementAt(4)+"</totalTrack>");
                fout.println("</job>");
            }
            fout.println("</Madagascar>");
            fout.flush();
            fout.close();
            
            
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
    }
}
