/*
* Title			:   Attribute.java
* Package		:   service.cubefs
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Almacena los atributos de un archivo.
*          
*                           		
* Created on            :   14/07/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package service.cubefs;

/**
 * Proporciona facilidades para tratar con los atributos de un archivo
 * @author victor Puertas.
 */
public class Attribute {
    
    /**
     * Constante que establece el permiso de descarga para el usuario.
     */
    static final public byte SET_U_DOWNLOAD = 32;
    /**
     * Constante que establece el permiso de borrado para el usuario.
     */
    static final public byte SET_U_ERASE    = 16;
    
    /**
     * Constante que establece el permiso de descarga para el grupo.
     */
    static final public byte SET_G_DOWNLOAD = 8;
    /**
     * Constante que establece el permiso de borrado para el grupo.
     */
    static final public byte SET_G_ERASE    = 4;
    
    /**
     * Constante que establece el permiso de descarga para el usuarios no pertenecientes al grupo ni propietarios.
     */
    static final public byte SET_O_DOWNLOAD = 2;
    /**
     * Constante que establece el permiso de borrado para el usuarios no pertenecientes al grupo ni propietarios.
     */
    static final public byte SET_O_ERASE    = 1;
    
    /**
     * Constante que establece el contenido como cifrado.
     */
    static final public byte SET_CIPHER     = (byte)128;
    /**
     * Constante que establece el contenido como privado.
     */
    static final public byte SET_PRIVATE    = 64;
   
    
    /**
     * Constructori de la clase.
     */
    public Attribute() {}
    
    
    /**
     * Indica si el contenido es desacargable por el propietario.
     */
    static public boolean U_download(byte mascara) { if((mascara & SET_U_DOWNLOAD)>0) return true; else return false; }
    /**
     * Indica si el contenido es borrable por el propietario.
     */
    static public boolean U_erase(byte mascara)    { if((mascara & SET_U_ERASE)>0) return true; else return false; }
    
    /**
     * Indica si el contenido es desacargable por el grupo.
     */
    static public boolean G_download(byte mascara) { if((mascara & SET_G_DOWNLOAD)>0) return true; else return false; }
    /**
     * Indica si el contenido es borraable por el grupo.
     */
    static public boolean G_erase(byte mascara)    { if((mascara & SET_G_ERASE)>0) return true; else return false; }
    
    /**
     * Indica si el contenido es desacargable por el resto de usuarios.
     */
    static public boolean O_download(byte mascara) { if((mascara & SET_O_DOWNLOAD)>0) return true; else return false; }
    /**
     * Indica si el contenido es borrable por el resto de usuarios.
     */
    static public boolean O_erase(byte mascara)    { if((mascara & SET_O_ERASE)>0) return true; else return false; }
    
    /**
     * Comprueba si el contenido es cifrado.
     */
    static public boolean iscipher(byte mascara)   { if((mascara & SET_CIPHER)>0) return true; else return false;}
    /**
     * Comprueba si el contenido es privado.
     */
    static public boolean isprivate(byte mascara)  { if((mascara & SET_PRIVATE)>0) return true; else return false; }
    
}
