/*
 * CubeFSGC.java
 *
 * Created on 5 de octubre de 2007, 13:34
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package service.cubefs;

import cube.common.pastry.NullContinuation;
import java.io.File;
import java.util.Iterator;
import rice.Continuation;
import rice.p2p.util.ImmutableSortedMap;
import rice.persistence.PersistentStorage;

/**
 * Recolector de fragmentos cuyo tiempo de vida ha expirado.<br>
 * Funcionamiento: Posee un periode determinado por el parametro refreshTime que 
 * determina el tiempo entre cada iteracion. Ademas de estos, se puede determinar
 * el numero maximo de fragmentos que se pueden borrar en cada iteracion. De esta forma
 * se intenta que no se pierdan muchos recursos computacionales en las pasadas en los casos
 * en los que se almacenen gran cantidad de fragmentos.<br><br>
 *
 * <b>Por defecto la longitud de la cola es 200 fragmentos y el tiempo de refresco 3600000 milisegundos</b>
 * (1 hora).
 *
 * @author victor Puertas
 */
public class CubeFSGC extends Thread {
    //Periodo del entre cada iteracion:
    private long refreshTime = 3600000;
    //Numero maximo de fragmentos a borrar en cada iteracion:
    private int queueSize = 200;
    /*Indice del ultimo fragmento procesado. Mediante este podemos continuar en la siguiente
    iteracion.*/
    private int index = 0;
    //Sistema de almacenamiento fisico de Past:
    private PersistentStorage storage = null; 
    //Condicion de parada de la hebra:
    private boolean stop = false;
    //Total de fragmentos recolectados:
    private long totalTrackRecolected = 0;
    
    /**
     * Constructor de la clase. La inicia con los valores por defecto.
     * @param storage Sistema de almacenamiento fisico de fragmentos de Past.
     */
    public CubeFSGC(PersistentStorage storage)
    { 
        this.storage = storage; 
    }
    
    /** 
     * Constructor de la clase.
     * @param refreshTime Tiempo transcurrido en milisegundos entre cada una de las iteraciones del recolector.
     * @param Numero maximo de fragmentos a borrar en cada iteracion.
     */
    public CubeFSGC(PersistentStorage storage, long refreshTime, int queueSize) 
    { 
        this.storage = storage; this.refreshTime = refreshTime; this.queueSize = queueSize;
    }
    
    public void run()
    {
        System.out.println("Iniciado el recolector de fragmentos caducados.");
  
        while(!stop)
        {
            //Numero de archivos borrados en la iteracion actual:
            int count_deletes = 0;
            
            
            long valor = cube.common.util.util.getUnixTime()-CubeFS_impl.TIME_OUT_DATA;
            rice.p2p.util.ImmutableSortedMap objectos_caducados = (ImmutableSortedMap) storage.scanMetadataValuesHead(valor);
            
            Iterator<rice.pastry.Id> expirados = objectos_caducados.keySet().iterator();
            
            while(count_deletes<queueSize && expirados.hasNext())
            {
                rice.pastry.Id myid = expirados.next();
                storage.unstore(myid, new NullContinuation());
                count_deletes++;
            }
            
            totalTrackRecolected+=count_deletes;
            //Bella durmiente:
           
            try { sleep(refreshTime); } catch (InterruptedException ex) {
                System.err.println("Recolector de fragmentos INTERRUMPIDO y FINALIZADO.");
                stopGC();
            }
           
        }
    }
    
    /**
     * Numeor de fragmentos recolectados hasta el momento.
     */
    public long gerRecolectedTrack(){ return totalTrackRecolected; }
    
    /**
     * Detiene el recolector de fragmentos caducados.
     */
    public void stopGC(){ stop = true; }
}
