/*
* Title			:   CubeFS_impl.java
* Package		:   service.cubefs
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Implementacion de Cube File Service.
*                           		
* Created on            :   21/07/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package service.cubefs;

import cube.common.logger.LogZero;
import cube.common.logger.NoActiveLoggerException;
import cube.common.pastry.Ring;
import cube.common.postman.Postman;
import cube.common.services.Impl_service;

import cube.common.util.util;
import java.io.*;

import java.util.AbstractMap;
import java.util.HashMap;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;
import rice.Continuation;
import rice.environment.Environment;
import rice.p2p.commonapi.Id;
import rice.p2p.past.Past;

import rice.p2p.past.PastPolicy;
import rice.pastry.PastryNode;
import rice.pastry.commonapi.PastryIdFactory;
import rice.persistence.Cache;
import rice.persistence.EmptyCache;
import rice.persistence.LRUCache;
import rice.persistence.MemoryStorage;
import rice.persistence.PersistentStorage;
import rice.persistence.Storage;
import rice.persistence.StorageManagerImpl;
import java.security.cert.Certificate;

/**
 * Implementacion deo Cube File Service.
 * @author victor Puertas.
 */
public class CubeFS_impl extends  Impl_service implements Runnable{
    
    //Directorio en donde almacenar los fragmentos de datos:
    private String root_directory;
    //Subdirectorio para almacenar fragmentos publicos:
    static private final String sub_root_directory_public = "public";
    //Subdirectorio para almacenar fragmentos privados:
    static private final String sub_root_directory_private = "private";
    //Indica el tipo de anillo: publico o privado: PUBLIC_RING = 0.
    private byte sortRing = 0; 
    /**
     * Persistencia TOTAL de los datos.
     */
    static public final byte PERSISTENT_TOTAL = 0;
    /**
     * Persistencia TEMPORAL de los datos. Tiempo de expiracion de los datos
     * determinado por <b>TIME_OUT_DATA</b>.
     */
    static public final byte PERSISTENT_TEMPORAL = 1;
    //Almacena el objeto que administra la cache:
    private Cache cache;
    //Longitud de la cache:
    private int cache_long = 0;
    //Lugar en el que implementar la cache: memoria o discoduro:
    private Storage cache_storage = null;
    //Representa al nodo en el anillo:
    private PastryNode node = null;
    //Directorio en el que almacenar la cache:
    private String cache_path = null;
    
    //Certificado del nodo
    private Certificate certNode;
 
    //Clase que representa al archivo de estado.
    private File sfFile;
    //Nombre del fichero de estado:
    private final String STATE_FILE_NAME = "cubeFS.st";
    /*Mapa que almacena la informacion de estado de cada uno de los archivos
    que hay en proceso. Esto permite retomar las tareas por donde quedaron. Esta clase es serializada
    al terminar la sesion de servicio.*/
    private  HashMap<String, Data_FileState> archivos;
    
    //Control de tareas:
    private Task serviceTask = new Task();
    
    //Recolector de fragmentos caducados:
    private CubeFSGC trackGC;
    //Almacena el objeto past:
    private Past past;
    //Numero de replicas de cada fragmento:
    private int replicas = 3;
    //Implementacion del sistema de almacenage de fragmentos:
    private StorageManagerImpl storage_manager = null;
    
    //Sistema para almacenar los fragmentos en el disco duro:
    private PersistentStorage almacen = null;
    
    /**
     * Longitud en bytes de un fragmento.
     */
    static public final int TRACK_LONG = 262144;
    
    //Espacio cedido en el disco duro (expresado en bytes):
    private long hd_space = 53687091200L;
    
    /**
     * Determina el tiempo maximo de vida de un fragmento con persistencia temporal.
     * El valor actual es de 15 dias. 24 h = 86400000 milisegundos.
     */
    static public final long TIME_OUT_DATA = 86400000 * 15; //15 dias.
    /**
     * Tiempo entra cada iteracion del la hebra que controla los fragmentos que han caducado.
     * Expresado en milisegundos.
     **/
    private long time_thread_data_expire = 3600000; //Cada hora.
       
    //Logger de la clase:
    static private Logger mylog;
    //Nombre del logger:
    static private final String logger_name = "service.CubePlatform.Ring";
    
      static
    {
        //Iniciamos el log para la clase:
        try { mylog = LogZero.CreateSubLogger(logger_name); } catch (NoActiveLoggerException ex) {}
    }
   
    /**
     * No hace nada. Se emplea para informacion del paquete por ejemplo.
     */
    public CubeFS_impl() {}
    
    /**
     * Constructor que carga opciones de configuracion del archivo de configuracion y 
     * permite determinar el tipo de anillo al que se accede.
     * @param sort_ring Tipo de anillo. Emplear las constantes <B>Ring.PUBLIC_RING</B> y<B> Ring.PRIVATE_RING</B>.
     * @param node Referencia al nodo en el anillo.
     */
    public CubeFS_impl(byte sort_ring, PastryNode node, Certificate certNode) throws IOException
    {
        System.out.println("Iniciando Cube File Service...");
        this.certNode = certNode;
        PastryIdFactory idf = new rice.pastry.commonapi.PastryIdFactory(node.getEnvironment());
        
        //Fichero de estado:
        sfFile = new File(Postman.getCubeDirectoryRoot()+File.separator+"cubeFS"+File.separator+"manageFS"+File.separator+STATE_FILE_NAME);
        System.out.println("CubeFS>>\tArchivo de estado: "+sfFile.toString());
        
        this.node = node;
        
        //Tipo de anillo: PUBLICO o PRIVADO.
        sortRing = sort_ring;
        root_directory = Postman.getCubeDirectoryRoot()+File.separator+"cubeFS"+File.separator+"hd"+File.separator;
        switch(sortRing)
        {
            case cube.common.pastry.Ring.PUBLIC_RING: 
                root_directory+=sub_root_directory_public+node.getId().toString();
               break;
            case cube.common.pastry.Ring.PRIVATE_RING: 
                root_directory+=sub_root_directory_private+node.getId().toString();
        }
        System.out.println("CubeFS>>\tDisco duro virtual montado sobre: "+root_directory);
      
        /*===================================================
         *  Sistema Cache
         *===================================================*/
        //Tamaño de la cache:
        cache_long = new Integer(System.getProperty("service.cubeFS.cacheSize"));
        
        //Directorio (path absoluto) en el que implementar la cache:
        cache_path = System.getProperty("service.cubeFS.cacheDirectory");
        String [] partes_path = cache_path.split(File.separator);
        //System.out.println("[0]"+partes_path[0]+"[1]"+partes_path[1]+"Separador: "+File.separator);
        if(partes_path[0].equals("{cuberoot}"))
        {
            cache_path = Postman.getCubeDirectoryRoot();
            for(int i = 1; i<partes_path.length; i++)
                cache_path+=File.separator+partes_path[i];  
        }
        
        System.out.println("Directorio cache: "+cache_path);
        //Lugar en el que se implementa la cache:
        if(System.getProperty("service.cubeFS.cacheStorage").equals("HD")) 
            try {cache_storage = new PersistentStorage(idf, cache_path, cache_long, node.getEnvironment());
            } catch (IOException ex) {
                System.err.println("CubeFS>>\tSe ha producido un error al crear el sistema cache tipo HD.");
                throw new IOException();
            }
        else cache_storage = new MemoryStorage(idf);
        
        //Algoritmo de cache:
        if(System.getProperty("service.cubeFS.cacheSystem").equals("LRU"))    
                cache = new LRUCache(cache_storage, cache_long, node.getEnvironment());
        else cache = new  EmptyCache(idf); //Sistema de cache apagado.  
        
        
         /*===================================================
         *  Almacen de fragmentos
         *===================================================*/
        //Factor de replica del fragmento:
        replicas =  new Integer(System.getProperty("service.cubeFS.replicas"));
        System.out.println("CubeFS>>\tFactor de replica: "+replicas);
        
        //Espacio cedido en el disco duro:
        hd_space = new Long(System.getProperty("service.cubeFS.spaceHD"));
        
        //Tiempo en milisegundos entre cada iteracion de la hebra que controla los fragmentos caducados en tiempo:
        time_thread_data_expire = new Long(System.getProperty("service.cubeFS.threadDataTempRefresh"));
        
        //Creamos el almacen de fragmentos:
        almacen = new PersistentStorage(idf, root_directory, hd_space, node.getEnvironment());
        storage_manager = new StorageManagerImpl(idf, almacen, cache);
    }
    
    /**
     *Comienza la ejecucion del servicio.
     *@param data parametros de inicio.
     */
    public void ServiceStart(Object data) 
    {
        trackGC = new CubeFSGC(almacen,time_thread_data_expire,200);
        trackGC.start();
        past = new service.cubefs.PastImpl(node,storage_manager, replicas, util.readPackageInfo(new CubeFS_impl(),"/service/cubefs/package_info/package_info.xml").internal_name);
        try 
        {
            //Cargamos la información sobre los trabajos pendientes de sesiones anteriores: 
            Load_task_file();
        } catch (IOException ex) {
            ex.printStackTrace();
        }        
    }
    
    /*Para el servicio. Se debe emplear este metodo para terminar de forma
     *correcta la ejecucion del servicio.
     */
    public void ServiceStop(byte tipo) 
    {
        stopAll();
        serviceTask.close();

        try {
            Save_task_file();
        } catch (IOException ex) {
          System.err.println("CubeFS>>\tError: al volcar información al fichero de estado.");
        }
    }
    
    /**
     * Pastry Enviroment de la conexion.
     */
    public Environment getNodeEnviroment() { return past.getEnvironment(); }
    
    /**
     * Numero de fragmentos recolectados.
     */
    public long getRelecectedFragment(){ return trackGC.gerRecolectedTrack(); }
    /**
     * Espacio en disco cedido a CubeFS.
     */
    public long getHdSpace(){ return hd_space; }
    
    /**
     * Tamaño de disco destinado a cache.
     */
    public long getCacheSpace(){ return cache_long;}
    
    /**
     * Inicializa TODOS los trabajos.
     */
    public void initializeAll()
    {
        Iterator<Data_FileState> iterador = archivos.values().iterator();
            
            while(iterador.hasNext())
            {
                FileManager FM = null;
                Data_FileState file_data = iterador.next();
                try {
                    //Encajamos los datos necesarios para poder llevar a cabo la accion:
                    Acciones_filemanager accion = new Acciones_filemanager();
                    accion.fileID = file_data.fileID.toString();
                    accion.ref_archivos = archivos;
                    accion.task = serviceTask;
                    //accion.ref_almacen_file_manager = almacen_file_manager;
                    
                    FM = new FileManager(new rice.pastry.commonapi.PastryIdFactory(node.getEnvironment()), past, certNode, serviceTask,file_data, accion, file_data.persistent);
                } catch (IllegalArgumentException ex) {
                    ex.printStackTrace();
                } catch (FileNotFoundException ex) {
                    ex.printStackTrace();
                } catch (IOException ex) {
                    ex.printStackTrace();
                } catch (ClassNotFoundException ex) {
                    ex.printStackTrace();
                }
               //Almacenamos el FileManager y lo podemos recuperar con la referencia, que es el fileID:
                            //almacen_file_manager.put(FM.GetFileID().toString(),FM);
               serviceTask.addFileManager(FM);
               FM.start();
            }
    }
    
    public void initializeOne(String fileID)
    {
        Data_FileState file_data = archivos.get(fileID);
        if(file_data==null)
        {
            System.err.println("CubeFS>>\tNo se encuentran los datos de estado de la tarea con fileID: "+fileID);
            return;
        }
        FileManager FM = null;
                
                try {
                    //Encajamos los datos necesarios para poder llevar a cabo la accion:
                    Acciones_filemanager accion = new Acciones_filemanager();
                    accion.fileID = file_data.fileID.toString();
                    accion.ref_archivos = archivos;
                    accion.task = serviceTask;
                    //accion.ref_almacen_file_manager = almacen_file_manager;
                    
                    FM = new FileManager(new rice.pastry.commonapi.PastryIdFactory(node.getEnvironment()), past, certNode, serviceTask,file_data, accion, file_data.persistent);
                } catch (IllegalArgumentException ex) {
                    ex.printStackTrace();
                } catch (FileNotFoundException ex) {
                    ex.printStackTrace();
                } catch (IOException ex) {
                    ex.printStackTrace();
                } catch (ClassNotFoundException ex) {
                    ex.printStackTrace();
                }
               
               serviceTask.addFileManager(FM);
               FM.start();
         
    }
    
    /**
     * Para TODOS los trabajos y libera recursos.
     */
    public void stopAll()
    {
        Iterator<FileManager> iterador = serviceTask.getFileManagers();
        while(iterador.hasNext())
        {
           iterador.next().close(Impl_service.TERMINATE);
        }
    }
    
    /**
     * Para el trabajo indicado como argumento.
     */
    public void stopOne(String fileID)
    {
        try
        {
            serviceTask.getFileManager(fileID).close(Impl_service.TERMINATE);
        }
        catch (NullPointerException ex){System.err.println("CubeFS>>\tNo existe una tarea asociada con el fileID: "+fileID);}
    }
    
    /**
     *Pausa todos los trabajos activos.
     */
    public void pauseAll()
    {
        Iterator<FileManager> iterador = serviceTask.getFileManagers();
        while(iterador.hasNext())
        {
           iterador.next().pause();
        }
    }   
    /**
     *Pausa el trabajo indicado.
     *@param fileID Identificador del trabajo a pausar.
     */
    public void pauseOne(String fileID)
    {
        try
        {
            serviceTask.getFileManager(fileID).pause();
        }
        catch (NullPointerException ex){System.err.println("CubeFS>>\tNo existe una tarea asociada con el fileID: "+fileID);}
    }
    /**
     * Reanuda todos los trabajos pausados.
     */
    public void continueAll()
    {
        Iterator<FileManager> iterador = serviceTask.getFileManagers();
        while(iterador.hasNext())
        {
           iterador.next().continue_();
        }
    }
    /**
     * Reanuda el trabajo pausado asociado con el fileID pasado como argumento.
     */
    public void continueOne(String fileID)
    {
        try
        {
            serviceTask.getFileManager(fileID).continue_();
        }
        catch (NullPointerException ex){System.err.println("CubeFS>>\tNo existe una tarea asociada con el fileID: "+fileID);}
    }
   
    /**
     * Borra los datos asociados a una tarea.
     * Esta llamada se emplea cuando se ha finalizado la tarea
     * y se desea eliminar su informacion de estado.<br>
     * <b>No usar si la tarea no se ha finalizado</b>
     */
    public void cleanInfoState(String fileID){ archivos.remove(fileID); }
    /**
     * Obtiene el estado de ejecucion del main-loop del FileManager.
     * @return FileManager.STATE_RUNNING, FileManager.STATE_PAUSE, FileManager.STATE_STOP
     * @return 255 si no existe un trabajo asociado con el fileID pasado como argumento.
     */
    public byte infoState(String fileID)
    { 
        try
        {
            return serviceTask.getFileManager(fileID).state();
        }
        catch (NullPointerException ex){}
        return (byte)255;
    }
    
    /**
     * Obtiene una estructura de datos con toda la informacion de la tarea.
     * @return null si no existe tarea asociada al fileID. De lo contrario devuelve 
     * una estructura con los datos de la tarea. 
     */
    public Data_FileState infoJob(String fileID)
    {
        try { return (Data_FileState) archivos.get(fileID).clone(); } 
        catch (NullPointerException ex){} return null; 
    }
    /**
     * Carga la informacion contenida en el archivo de control de tareas llamado cubeFS.st
     * y almacenado en /manageFS. Inicializa la estructura de datos: <B>archivos</B>.
     * @throws java.io.IOException Causas: El archivo es corrupto.<br>
     * Error al IO al extraer la informacion.<br>
     * El archivo no se encuentra.<br>
     */
    private void Load_task_file() throws IOException
    {
        FileInputStream sfIn = null;
        
        
        if(!sfFile.exists())
        {    
            try {
                sfFile.createNewFile();
            } catch (IOException ex) {
                System.err.println("CubeFS>>\tError tipo I/O al crear el fichero de control de tareas de CubeFS.");
                throw new IOException();
            }
            archivos= new HashMap();
        }
        else
        {
            try {
                sfIn = new FileInputStream(sfFile);
            } catch (FileNotFoundException ex) {
                System.err.println("CubeFS>>\tEl fichero de control de tareas ha desaparecido de forma subita.");
                //System.err.println("Error al abrir el fichero de estado");
                throw new IOException();
            }
            if(sfFile.length()<1) {archivos= new HashMap(); return;}
           
            
            try { archivos = (HashMap<String, Data_FileState>) util.unSerializeObject(sfIn); }
            catch (ClassNotFoundException ex)
            {
                System.err.println("CubeFS>>\tArchivo de control de tareas corrupto.");
                throw new IOException();
            }
            catch (IOException ex) {
                System.err.println("CubeFS>>\tError tipo I/O al leer en el fichero de control de tareas.");
                throw new IOException();
            }
        }
    }
    
    /**
     * Almacena la informacion contenida en el archivo de control de tareas.
     * @throws java.io.IOException Causas: El archivo es corrupto.<br>
     * Error al IO al escribir la informacion.<br>
     * El archivo no se encuentra.<br>
     */
    private void Save_task_file() throws IOException
    {
        FileOutputStream sfOut = null;

        try {
                sfFile.delete();
                sfFile.createNewFile();
                sfOut = new FileOutputStream(sfFile);
            } catch (FileNotFoundException ex) {
                System.err.println("CubeFS>>\tEl fichero de control de tareas recien creado ha desaparecido.");
                //System.err.println("Error con el archivo de estado.");
                throw new IOException();
            }
        
        util.serializeObject(sfOut,archivos);
    }
    
     /**
     *Inicia una subida de archivo.
     *@param workfile Archivo en el que se almacenaran los datos obtenidos.
     *@param fsfile Archivo FS del que obtener la informacion de descarga.
     *@param metadatos Metadatos asociados al archivo.
     *@param persistent Tipo de persistencia.
     *@return FileID del archivo. Este identificador es necesario para poder pedir informacion<BR>
     *acerca del estado del archivo.
     */
    public String up_file(String workfile, String fsfile, MetadataFile metadatos, byte persistent) 
    {
        if(persistent!=PERSISTENT_TOTAL && persistent!=PERSISTENT_TEMPORAL) return null;
        try 
        {
           if(!new File(workfile).exists()) {return null;};
           Data_FileState myoperation = new Data_FileState(FileManager.STATE_UP, metadatos, TRACK_LONG,(byte)0, workfile, fsfile, persistent);
           Acciones_filemanager accion = new Acciones_filemanager();
           accion.ref_archivos = archivos;
           accion.task = serviceTask;
           //accion.ref_almacen_file_manager = almacen_file_manager;
           
           FileManager FM = new FileManager(new rice.pastry.commonapi.PastryIdFactory(node.getEnvironment()), past, certNode, serviceTask, myoperation, accion, persistent);
          
           if(!archivos.containsKey(FM.GetFileID().toString()))
           {
                archivos.put(FM.GetFileID().toString(),myoperation);
                serviceTask.addFileManager(FM);
           }
           
           if(FM.getState() == java.lang.Thread.State.NEW)
           {
               serviceTask.addFileManager(FM);
               FM.start();
           }
           return FM.GetFileID().toString();
           
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IllegalArgumentException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        }
        return null;
    }
    
    /**
     *Inicia una descarga de archivo.
     *@param workfile Archivo en el que se almacenaran los datos obtenidos.
     *@param fsfile Archivo FS del que obtener la informacion de descarga.
     *@return FileID del archivo. Este identificador es necesario para poder pedir informacion<BR>
     *acerca del estado del archivo.
     */
    public String down_file(String workfile, String fsfile)
    {
         try {
                new File(workfile).createNewFile();
                Data_FileState myoperation = new Data_FileState(FileManager.STATE_DOWN, null, TRACK_LONG, (byte)0,workfile,fsfile, (byte)0);
                Acciones_filemanager accion = new Acciones_filemanager();
                accion.ref_archivos = archivos;
                accion.task = serviceTask;
                //accion.ref_almacen_file_manager = almacen_file_manager;
                FileManager FM = new FileManager(new rice.pastry.commonapi.PastryIdFactory(node.getEnvironment()), past, certNode, serviceTask, myoperation, accion, PERSISTENT_TOTAL);
               
               // if(!archivos.containsKey(FM.GetFileID().toString()))
                //{
                    
                    archivos.put(FM.GetFileID().toString(),myoperation);
                    serviceTask.addFileManager(FM);
               // }
                if(FM.getState() == java.lang.Thread.State.NEW)
                {
                    serviceTask.addFileManager(FM);
                    FM.start();
                }
                return FM.GetFileID().toString();
                
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IllegalArgumentException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        }
         return null;
    }
    
       
    /**
     * Facilita la escritura de mensajes en el Logger solo si este está activo.
     * 
     */
    static private void WriteLog(Level nivel,String mensaje) {if (LogZero.Active()){ synchronized(mylog){mylog.log(nivel,mensaje);}}}    

    public void run() {
    }

   
}

