/*
* Title			:   Operation.java
* Package		:   service.cubefs
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Alberga información sobre un archivo con el que esta
*                           trabajando el File Manager.
*          
*                           		
* Created on            :   08/07/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package service.cubefs;

import java.io.Serializable;
import java.util.TreeSet;


/**
 * Almacena información de estado sobre el archivo asociado File Manager.
 * La clase es serializable debido a que su contenido es volcado a un fichero de 
 * estado.
 * @author victor Puertas
 */
public class Data_FileState implements Serializable, Cloneable
    {
       /**
        * Version de la clase.
        */
        static private final long serialVersionUID = 10000;

        /**
         * Identificador de archivo.
         */
        public FileID fileID;
        /**
         * Metadatos asociados al archivo.
         */
        public MetadataFile mymetadata;
        /**
         * Longitud del fragmento.
         */
        public int track_long;
        /**
         * Permisos del archivo, en caso de tratarse de un anillo privado.
         */
        public byte privileges = 0;
        /**
         * Archivo sobre el que se trabaja.
         */
        public String  workfile;
        /**
         * Archivo FS asociado.
         */
        public String fsFile;        
        /**
         * Numero total de fragmentos.
         */
        public long totalTrack = 0;
        /**
         * Numero de fragmentos subidos/descargados.
         */
        public long trackok = 0;
        /**
         * Numero de fragmentos erroneos.
         */
        public long trackbad = 0;
        /**
         * Copias insertadas en el anillo.
         */
        public long backup_tracks = 0;
        
        /**
         * Almacena el conjunto de fragmentos/indices problematicos:
         */
        public TreeSet conjunto = new TreeSet();
        /**
         * Estado del trabajo.
         */ 
        private byte state = 13; //STATE_TERMINATE.
        /**
         * Indice del fragmento actual en la tarea. No se emplea en los 
         * fragmentos erroneos.
         */
        public int indice = 0;
        
        /**
         * Tipo de persistencia.
         */
        byte persistent = 0;
        
      /**
       * Contructor de la clase.
       */  
     public Data_FileState(byte state, MetadataFile mymetadata, int track_long, byte privileges, String  workfile, String fsFile, byte persistent)
      {
          this.mymetadata = mymetadata; this.track_long = track_long; this.privileges = privileges; this.workfile = workfile;
          this.fsFile = fsFile; this.state = state; this.persistent = persistent;
      }
     
     /**
      * Obtiene el estado del trabajo.
      */
     synchronized public byte getState() { return state; } 
     
     /**
      * Cambia el estado en el que se encuentra la tarea.
      */
     synchronized public void setState(byte newState) { state = newState; }
     
    /**
     * Permite clonar el objeto.
     */
     public Object clone()
      {
            Data_FileState obj = null;
        try {
            obj = (Data_FileState) super.clone();
        } catch (CloneNotSupportedException ex) {
            //Solo en depuracion
            System.err.println("No se puede clonar el data_fileState.");
        }
            obj.workfile = new String(this.workfile);
            obj.fsFile = new String(this.fsFile);
            obj.conjunto =  (TreeSet) this.conjunto.clone();
            obj.fileID = (FileID) this.fileID.clone();
            obj.mymetadata = (MetadataFile) this.mymetadata.clone();
            return obj;
      }
    }



