/*
* Title			:   FileID.java
* Package		:   service.cubefs
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Almacena un identificador de archivo (FileID).
*                           		
* Created on            :   9/06/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package service.cubefs;
import cube.common.security.Cryptography;
import service.cubefs.exception.BadIdentifier;



/**
 * Almacena un identificador de archivo. Este es prodicido normalmente por
 * FileIDFactory como resultado de obtener un resumen sobre un contenido
 * determinado.
 */
public class FileID implements java.io.Serializable, Cloneable {
    
    //Alamacena el identificador de archivo: FileID.
    private byte [] fileID;
    //Longitud del identificador en bytes:
    static private final byte SIZE_FILE_ID = 20;
    
    /**
     * Permite crear un FileID a partir del conjunto de bytes obtenido por algun generador
     * de identificadores de archivo como por ejemplo FileIDFactory. Tambien se pueden
     * conseguir aplicando un algoritmo suma como SHA-1 con el tamaño adecuado.
     * Si el conjunto de bytes introducidos no posee el tamaño adecuado, el FileID
     * queda a <B>null</B>.
     * @param key Conjunto de bytes que conforman el FileID.
     * @throws service.cubefs.exception.BadIdentifier El identificador esta mal formado. Motivo: No cumplir con la longitud o el valor es nulo.
     */
    public FileID(byte [] key) throws BadIdentifier { if(key.length != SIZE_FILE_ID || key == null) throw new BadIdentifier(); fileID = key;}
    
    /**
     * Constructor de copia que permite crear un FileID a partir de otro.
     * @param obj FileID que se desea clonar.
     */
    public FileID(FileID obj) { fileID = obj.fileID;}
    
    /**
     * Permite comparar un FileID con otro.
     * @return TRUE  - Los identificadores de archivo son iguales.
     * FALSE - Lo contrario.
     * @param obj FileID con el que realizar la comparacion.
     */
    public boolean equal(FileID obj)
    {
        if (obj==null) return false;
        /*if(this.fileID.length != obj.fileID.length) return false;
        for(int cont=0;cont<obj.fileID.length;cont++)
        {
            if(obj.fileID[cont]!=this.fileID[cont]) return false;
        }
        return true;*/
        return java.util.Arrays.equals(fileID, obj.fileID);
    }
    
    /**
     * Obtiene la representación en bytes del identificador de archivo.
     * @return Conjunto de bytes.
     */
    public byte [] GetBytes() { return fileID; }
    
    /**
     * Devuelve el tamaño en bytes del FileID.
     * @return Numero de bytes que posee el FileID.
     */
    static public int GetSize(){return SIZE_FILE_ID;}
    
    /**
     * Devuelve una cadena de texto con el FileID en formato hexagecimal.
     * @return FileID en formato string.
     */
    public String toString() {return rice.p2p.util.MathUtils.toHex(fileID);}
    
    /**
      * Permite clonar el objeto.
      */
     public Object clone()
     {
         FileID obj = null;
        try {
            obj = (FileID) super.clone();
        } catch (CloneNotSupportedException ex) {
            //Solo en depuracion
            System.err.println("No se puede clonar el FileID.");
        }
         return obj;
     }
}
