/*
* Title			:   FileIDFactory.java
* Package		:   service.cubefs
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Genera identificadores de fichero unicos.                          		
* Created on            :   19/06/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package service.cubefs;

import cube.common.logger.LogZero;
import cube.common.logger.NoActiveLoggerException;
import cube.common.postman.Postman;
import cube.common.security.Cryptography;
import cube.common.util.util;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import java.security.cert.CertificateEncodingException;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import service.cubefs.exception.BadIdentifier;

/**
 * Genera identificadores de fichero unicos a partir de unos parametros. 
 * Si se le proporciona un certificado, los genera empleando su clave publica.
 * De lo contrario empleado la marca de tiempo Unix + path del fichero + numero aleatorio.
 */
public class FileIDFactory {  
   
    /**
     * Genera identificadores de fichero (<I>fileID</I>) unicos para CubeFS.
     */
    public FileIDFactory() {}
    
    /**
     * Genera un <I>fileID</I> unico basandose en lo parametros descritos al inicio.
     * @param path Puede ser cualquier cadena de texto, pero normalmente se empleal el <B>path completo del 
     * fichero</B>. Si no se introduce nada, la cadena por defecto es <B>CubeFS</B>.
     * @return Identificador generado o <b>null</b> si ocurrio algun problema.
     */
    static public FileID GenerateID(Certificate cert, String path)
    {
        byte public_key [] = null;
        byte concatenado [] = null;
        byte aleatorio [];
        byte resultado [] = null;
        int indice;
        
        if (path == null) path = "CubeFS";
        
        //Obtener la clave publica:
        //java.security.cert.Certificate certificado = Postman.GetRefCryptography().getCertificate(Cryptography.alias_nodeCert);
        
        //if(cert == null) { WriteLog(Level.INFO,"No se puede generar el fileID: imposible obtener la clave publica del certificado de nodo."); return null; } 
        
        if(cert != null)
        {
            try {
                public_key = cert.getEncoded();
            } catch (CertificateEncodingException ex) {
                System.out.println("Se ha producido un error durante la lectura de la clave publica del certificado de nodo."); return null;
            }
        }
        
        //obtener numeros aleatorios:
        
        aleatorio = Cryptography.getRandom(10);
        if (aleatorio == null) {System.out.println("No se puede generar el fileID: imposible obtener el numero aleatorio."); return null; }
       
            
        //reservamos el tamaño necesario:
        if(cert != null)
        {
            concatenado = new byte [public_key.length + aleatorio.length * 2 + path.getBytes().length];
            indice = util.concatbytes(public_key, concatenado, 0, public_key.length);
            indice = util.concatbytes(aleatorio, concatenado, indice, aleatorio.length); 
            indice = util.concatbytes(path.getBytes(), concatenado, indice, path.getBytes().length); 
            
            //obtener numeros aleatorio 2:
        
            aleatorio = Cryptography.getRandom(10);
            if (aleatorio == null) {System.out.println("No se puede generar el fileID: imposible obtener el numero aleatorio."); return null; }
        
                util.concatbytes(aleatorio, concatenado, indice, aleatorio.length);
        }
        else
        {
            String fecha = Long.toHexString(new Date().getTime());
            concatenado = new byte [fecha.length() + aleatorio.length + path.getBytes().length];
            indice = util.concatbytes(fecha.getBytes(), concatenado, 0, fecha.getBytes().length);
            indice = util.concatbytes(aleatorio, concatenado, indice, aleatorio.length); 
            indice = util.concatbytes(path.getBytes(), concatenado, indice, path.getBytes().length); 
            
        }     
        
        //Obtener el resumen del contenido:
        try { resultado = Cryptography.digestSHA1(concatenado); } catch (NoSuchAlgorithmException ex) { return null; }
                  
        try { return new FileID(resultado); }    
        catch (BadIdentifier ex) { System.out.println("No se puede generar el fileID: imposible obtener el resumen."); return null; }
        
    }
}
