/*
* Title			:   FileManager.java
* Package		:   service.cubefs
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Gestiona los fragmentos de archivo.
*                           en el anillo.
*          
*                           		
* Created on            :   14/12/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package service.cubefs;

import cube.common.logger.LogZero;
import cube.common.logger.NoActiveLoggerException;
import cube.common.security.Cryptography;
import cube.common.services.Impl_service;
import cube.common.util.util;
import java.io.*;
import java.security.InvalidKeyException;
import java.security.cert.Certificate;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.crypto.SecretKey;
import rice.Continuation;
import rice.p2p.past.Past;
import rice.p2p.past.gc.GCPastImpl;
import rice.pastry.commonapi.PastryIdFactory;
import rice.tutorial.past.MyPastContent;
import service.cubefs.exception.*;


/**
 * Permite subir o extraer un archivo del anillo al que esta conectado CubeFS.
 * Para cada archivo con el que se desee operar se crea un gestor, FileManager que es el encargado
 * de crear, a su vez, un gestor de fragmentos, TrackManager que sera el encargado de obtener o escribir
 * fragmentos en el archivo de trabajo entendiendose este como el archivo que sera origen/destino de los 
 * datos subidos o extraidos del anillo.<br><br>
 * <B>Limitaciones de la version actual</B><br>
 * - No contempla la gestion de archivos privados.<br><br>
 * @author victor Puertas.
 */

/*
 *@todo Revisar el codigo para optimizarlo.
 */
public class FileManager extends Thread {
    
    //Estados del FileManager:
    
    /**
     * Descargadndo archivo.
     */
    static public final byte STATE_DOWN = 10;
    /**
     * Subiendo archivo
     */
    static public final byte STATE_UP = 11;
    /**
     * Trabajo finalizado
     */
    static public final byte STATE_TERMINATE = 13;
   
    //Gestor del fichero FS:
    private FileFS myFSfile;
    
    /**
     * Almacena la informacion de la tarea asociada
     * al FileManager. Es una estructura muy IMPORTANTE.
     */
    private Data_FileState FM_state;
    
    //Referencia a la implementacion PAST.
    private PastImpl dht;
    
    //FileManager Continuation: Permite pasar eventos surgidos.
    private Continuation command;
    
   /**
    * Generador de identificadores empleado en el anillo.
    * Permite generar los trackID al trackManager.
    */
    private PastryIdFactory IDFactory;
    
    //TrackManager.
    private TrackManager trackmanager;
    
    //Gestor de tareas del CubeFS:
    private Task taskManager;
   
    
    //Estados del main-loop:
    
    /**
     * Pausado.
     */
    static public final byte STATE_PAUSE = 14;
    /**
     * Ejecutandose
     */
    static public final byte STATE_RUNNING = 15;
    /**
     * Parado
     */
    static public final byte STATE_STOP = 16;
   /**
    * Indica si se pausa la tarea.
    */
    private Boolean pauseFileManager = false; 
   /**
    *Indica si se tiene que detener el main-loop:
    */
    private Boolean stopMainLoop = false;
    
    /**
     * Sube o baja archivos de un anillo.
     * @param IDFactory Generador de identificadores de Pastry.
     * @param DHT Implementacion de PAST.
     * @param certNode Certificado del nodo.<br>
     * El certificado puede ser nulo en cuyo caso los FileID se generan empleando numeros aleatorios.
     * @param taskmanagement Gestor de tareas y colas de CubeFS.
     */
    public FileManager(PastryIdFactory IDFactory, Past DHT, Certificate certNode, Task taskmanagement, Data_FileState datos, 
            Continuation command, byte persistent) throws FileNotFoundException, IOException, IllegalArgumentException, ClassNotFoundException {
  
        dht = (PastImpl) DHT;
        taskManager = taskmanagement;
        FM_state = datos;
        this.IDFactory = IDFactory;
        this.command = command;

        myFSfile= new FileFS(datos.fsFile);
               
        switch(FM_state.getState())
        {
            case STATE_DOWN:
      
                myFSfile.Open();
                
                //Rellenamos la informacion de estado de la tarea:
                FM_state.fileID     = myFSfile.Get_info_FileID();          
                FM_state.track_long = myFSfile.Get_info_TrackLong();
                FM_state.mymetadata = myFSfile.Get_info_metadata();
                FM_state.privileges = myFSfile.Get_info_privileges();
                FM_state.totalTrack = myFSfile.GetTotalTrack();
                
                trackmanager = new TrackManager(IDFactory,FM_state.privileges, FM_state.workfile, TrackManager.MODE_READ_WRITE, FM_state.fileID, 
                                                FM_state.track_long, persistent);
                break;
                
            case STATE_UP: 
                //La primera vez que se carga:            
                if(FM_state.fileID == null)
                {
                    //Generamos el identificador de archivo y creamos el fs file:
                    FM_state.fileID = FileIDFactory.GenerateID(certNode, datos.workfile); 
                    myFSfile.Create(FM_state.fileID, FM_state.track_long, FM_state.privileges, FM_state.mymetadata); 
                }
                else
                {
                    myFSfile.Open();
                    myFSfile.SetIndex(FM_state.indice);
                    
                }
                long sizeFile = new File(FM_state.workfile).length();
                FM_state.totalTrack = sizeFile/FM_state.track_long;
                if(sizeFile%FM_state.track_long>0) FM_state.totalTrack++;
                System.out.println("Total de fragmentos: "+FM_state.totalTrack);
                //Creamos el TrackManager:
                trackmanager = new TrackManager(IDFactory,FM_state.privileges, FM_state.workfile, TrackManager.MODE_READ, FM_state.fileID, FM_state.track_long, persistent);
                break;
                
                         
        }
        //Nombre del thread:
        this.setName("FileManager["+FM_state.fileID+"]");
        //Le pasamos a la accion el fileID para que puede realizar operaciones.
        ((Acciones_filemanager)command).fileID = FM_state.fileID.toString();
    }
    
   /**
    * Main loop
    */ 
   public void run()
   {
       byte theTrackID [] = new byte[20];
       CubeFSTrack track_insert = null;
       
       while(!stopMainLoop)
       {
           switch(FM_state.getState())
           {
               /*
                * SUBIENDO ARCHIVO:
                */
               case STATE_UP:
                   if(FM_state.indice<FM_state.totalTrack)
                   {
                       try{ track_insert = trackmanager.ReadTrack(FM_state.indice); }
                       catch(KeyCipherNotFound ex)
                       { 
                           FM_state.setState(STATE_TERMINATE); 
                           close(Impl_service.TERMINATE); 
                           command.receiveException(ex); 
                           return; 
                       } 
                       catch(InvalidKeyException ex)
                       { 
                           FM_state.setState(STATE_TERMINATE); 
                           close(Impl_service.TERMINATE); 
                           command.receiveException(ex); 
                           return; 
                       } 
                       
                        if(track_insert == null) 
                        {
                             System.err.println("El trackmanager ha obtenido un fragmento nullo.");
                            if(System.getProperty("service.cubeFS.ContinueInsertOnERROR").equals("OFF")) 
                            {
                                FM_state.setState(STATE_TERMINATE); 
                                System.err.println("FileManager>>Tarea "+FM_state.fileID.toString()+": el Track Manager obtuvo un fragmento nullo.");
                                close(Impl_service.TERMINATE);
                                command.receiveException(new IOException("Track Manager obtuvo un fragmento nulo."));
                                return;
                            }
                            FM_state.indice++;
                            break;
                        }
                        
                       taskManager.addTrackUp(FM_state.fileID.toString()); //Lo añadimos a la cola de subida.

                       synchronized(dht) { dht.insert(track_insert, buildTaskUp(track_insert.getId().toByteArray())); }
                       
                       FM_state.indice++;
                       if (FM_state.indice==FM_state.totalTrack)
                       {
                           while(taskManager.getTrackInQueue(FM_state.fileID.toString())>0)
                           {
                                try { this.sleep(1000); } catch (InterruptedException ex) {}
                           }
                             
                           FM_state.indice++; //Para detectar cuando cambiamos de estado y podamos operar bien con el contador badtrack.
                       }
                       break;
                   }
                   //Fragmentos problematicos:
                   if(FM_state.conjunto.size()>0)
                   {    
                       int indice = (Integer) FM_state.conjunto.first();
                       FM_state.conjunto.remove(indice);
                       
                       try { track_insert = trackmanager.ReadTrack( indice ); }  
                       catch(KeyCipherNotFound ex)
                       { 
                           FM_state.setState(STATE_TERMINATE); 
                           close(Impl_service.TERMINATE); 
                           command.receiveException(ex); 
                           return; 
                       } 
                       catch(InvalidKeyException ex)
                       { 
                           FM_state.setState(STATE_TERMINATE); 
                           close(Impl_service.TERMINATE); 
                           command.receiveException(ex); 
                           return; 
                       }     
                          
                       if(track_insert == null) 
                       {
                            if(System.getProperty("service.cubeFS.ContinueInsertOnERROR").equals("OFF")) 
                            {
                                FM_state.setState(STATE_TERMINATE); 
                                System.err.println("FileManager>>Tarea "+FM_state.fileID.toString()+": el Track Manager obtuvo un fragmento nullo.");
                                close(Impl_service.TERMINATE);
                                command.receiveException(new IOException("Track Manager obtuvo un fragmento nulo."));
                                return;
                            }
                            FM_state.indice++;
                            break;
                       }
                       
                       BoxSyncr sincronizador = new BoxSyncr();
                       taskManager.addTrackUp(FM_state.fileID.toString());
                       synchronized(dht){ dht.existsInOverlay(track_insert.getId(), buidTaskTrackExist(sincronizador)); }
                       
                       sincronizador.Stop();    //Esperamos resultados...
                      
                       if(!(Boolean)sincronizador.aux)    //Fragmento NO insertado.
                            synchronized(dht){ dht.insert(track_insert, buildTaskUp(track_insert.getId().toByteArray())); }
                       else
                       {
                            synchronized(FM_state) { FM_state.trackok++; }
                            taskManager.subTrackUp(FM_state.fileID.toString());
                            try { myFSfile.SetTrackID(track_insert.getId().toByteArray()); }
                            catch (IOException ex) 
                            {
                                System.err.println("FileManager>>Tarea "+FM_state.fileID.toString()+": Error al escribir el fragmento en archivo FS. "+
                                                   rice.p2p.util.MathUtils.toHex(track_insert.getId().toByteArray()));
                            }
                       }
                       
                       break;
                   }
                   
                   
                   /*Esperamos a que terminen los fragmentos solicitados por si alguno da 
                     problemas, pueda continuar intentandolo.*/

                   while(taskManager.getTrackInQueue(FM_state.fileID.toString())>0)
                   {
                        try { this.sleep(1000); } catch (InterruptedException ex) {}
                   }
                                      
                   FM_state.setState(STATE_TERMINATE); 
                   command.receiveResult(new ReportFile(FM_state.fileID, FM_state.trackok, FM_state.trackbad, FM_state.backup_tracks, FM_state.fsFile));  
                  break; 
               /*
                * DESCARGANDO ARCHIVO:
                */   
               case STATE_DOWN:
                   
                   if(FM_state.indice<FM_state.totalTrack)
                   {
                       
                       try { theTrackID = myFSfile.GetTrackID(FM_state.indice); }
                       catch (EOFException ex)
                       { 
                                FM_state.setState(STATE_TERMINATE); 
                                System.err.println("FileManager>>Tarea "+FM_state.fileID.toString()+": fichero fs corrupto.");
                                close(Impl_service.TERMINATE);
                                command.receiveException(new IOException("Fichero fs corrupto."));
                                return; 
                       }
                       catch (IOException ex) 
                       {
                            
                            if(System.getProperty("service.cubeFS.ContinueInsertOnERROR").equals("OFF")) 
                            {   
                                FM_state.setState(STATE_TERMINATE); 
                                System.err.println("FileManager>>Tarea "+FM_state.fileID.toString()+": error I/O.");
                                close(Impl_service.TERMINATE);
                                command.receiveException(ex);
                                return;
                            }
                            FM_state.indice++;
                            break;    
                       }
                       
                       taskManager.addTrackDown(FM_state.fileID.toString());
                       synchronized(dht) { dht.lookup(IDFactory.buildId(theTrackID), buildTaskDown(theTrackID)); }
                      
                       FM_state.indice++;
                       if (FM_state.indice==FM_state.totalTrack)
                       {
                           while(taskManager.getTrackInQueue(FM_state.fileID.toString())>0)
                           {
                                try { this.sleep(1000); } catch (InterruptedException ex) {}
                           }
                           FM_state.indice++;
                       }
                       break;
                   }
                   
                   
                   
                   
                   
                   //Fragmentos problematicos:
                   if(FM_state.conjunto.size()>0)
                   {
                       theTrackID = (byte[]) FM_state.conjunto.first();
                       FM_state.conjunto.remove(theTrackID);
                       
                       taskManager.addTrackDown(FM_state.fileID.toString());
                       synchronized(dht) { dht.lookup(IDFactory.buildId(theTrackID), buildTaskDown(theTrackID)); }
                       
                       break;
                   }
                   
                   /*Esperamos a que terminen los fragmentos solicitados por si alguno da 
                     problemas, pueda continuar intentandolo.*/

                   while(taskManager.getTrackInQueue(FM_state.fileID.toString())>0)
                   {
                        try { this.sleep(1000); } catch (InterruptedException ex) {}
                   }
                   
                   FM_state.setState(STATE_TERMINATE); 
                   command.receiveResult(new ReportFile(FM_state.fileID, FM_state.trackok, FM_state.trackbad, FM_state.backup_tracks, FM_state.fsFile));  
                  break;
                  
               /*
                * TAREA FINALIZADA:
                */  
               case STATE_TERMINATE:
                   close(Impl_service.TERMINATE);
                   return;
           }
            try { this.sleep(500); } catch (InterruptedException ex) {}
           //¿Se ha solicitado pausar la tarea?
           synchronized(this)
           {
               if(pauseFileManager) {try { this.wait(); } catch (InterruptedException ex) {}}
           }
       }
   }
   
   /**
     * Cierra el FileManager y libera recursos, cerrando el archivo fs empleado. <B>Siempre se debe utilizar este 
     * metodo para evitar errores en los datos. IMPORTANTE: es recomentable emplear TERMINATE en lugar de TERMINATE NOW
     * para que no se produzcan perdidas de informacion.</B> 
     * @return Clase con el estado de la operacion que esta realizando el FileManager. La clase
     * es serializable pues la implementacion del servicio la volcara a archivo.
     * @param Indica el tipo de detencion. Valores posibles del parametro: TERMINATE Y TERMINATE_NOW DE Impl_service.
     * Con el segundo valor se cierra el FileManager de forma brusca cortando todas la tareas pendientes.
     */
    public Data_FileState close(byte terminate)
    {
        synchronized(stopMainLoop) {stopMainLoop = true; }
        continue_();
         
        //Esperamos a que terminen los fragmentos lanzados a menos que se indique lo contrario:
        if(terminate != Impl_service.TERMINATE_NOW)
        {
             while(taskManager.getTrackInQueue(FM_state.fileID.toString())>0)
             {
                try { this.sleep(1000); } catch (InterruptedException ex) {}
             }
        }   
        trackmanager.close();
        myFSfile.Close();    
        System.gc();
        
        System.out.println("FileManager>>Tarea "+FM_state.fileID.toString()+ " cerrada con éxito.");
        return FM_state;
    }
    
    /**
     * Obtiene el fileID del archivo.
     * @return Identificador de archivo.
     */
    public FileID GetFileID(){ return FM_state.fileID;}
    
    /**
     * Pausa las operaciones del FileManager.
     */
    synchronized public void pause(){  pauseFileManager = true; }
    
    /**
     * Reanuda las operaciones del FileManager previamente pausado.
     */
    synchronized public void continue_()
    { 
        if(pauseFileManager) 
        { 
            pauseFileManager = false; 
            notifyAll(); 
        } 
    }
    
    /**
     * Consulta el estado del main-loop.
     * @return: STATE_RUNNING, STATE_PAUSE, STATE_STOP
     */
    public byte state()
    { 
        if(stopMainLoop)
            return STATE_STOP;
        else
          if(pauseFileManager) return STATE_PAUSE;
          else return STATE_RUNNING;
    }
    
    /**
     * Establece la clave de cifrado/descifrado para el contenido del archivo.
     * @param key Clave de cifrado.
     */
    public void SetKey(SecretKey key){ trackmanager.SetKey(key); }
   
   /**
    * Contruye las tareas a realizar con un fragmento en 
    * la subida del mismo.
    */
   private Task_up buildTaskUp(byte[] trackID)
   {
         Task_up task = new Task_up();
         task.trackID = trackID;
         task.indice = FM_state.indice;
         task.ref_FM_state = FM_state;
         task.fileFS = myFSfile;
         task.task = taskManager;
         return task;
   }
    
   /**
    * Contruye las tareas a realizar con un fragmento en 
    * la subida del mismo.
    */
   private Task_down buildTaskDown(byte[] trackID)
   {
         Task_down task = new Task_down();
         task.trackID = trackID;
         task.refTrackManager = trackmanager;
         task.ref_FM_state = FM_state;
         task.refCommand = command;
         task.task = taskManager;
         return task;
   }
   
   /**
    * Contruye la tarea con las acciones a realizar cuando se 
    * comprueba si un fragmento esta insertado en el anillo.
    */
   private Task_Track_Exist buidTaskTrackExist(BoxSyncr syncr)
   {
       Task_Track_Exist aux = new Task_Track_Exist();
       aux.syncr = syncr;
       return aux;
   }
   
}//Fin de la clase.


/**
     * Contiene las tareas a realizar tras una insercion. Funciona a modo de evento.
     * Cuando la insercion se produce, Past emplea esta clase. Para que pueda operar
     * de forma correcta se deben asignar valores a los campo: fragmento, indice, instanceState.
     * Al ser una clase interna  no emplea metodos para encapsular las variables.
     * Requiere:<br>
     * -trackID.<br>
     * -Referencia al FM_state.<br>
     * -Indice del fragmento.<br>
     * -Referencia al controlador del fichero FS.<br>
     * -Referencia al gestor de tareas.<br>
     
     */    
    class Task_up implements Continuation
    {  
        /**
        * Indentificador del fragmento.
        */
        byte trackID [];
       /**
        * Indice del fragmento.
        */
        int indice = 0;
        /**
         * Referencia a la estructura con informacion del trabajo.
         */
        Data_FileState ref_FM_state;
        /**
         * Referencia al archivo FS.
         */
        FileFS fileFS;
        /**
         * Referencia del Supervisor de tareas:
         */
        Task task;
       
        public void receiveResult(Object object) 
        {
              //Sacar el fragmento de la cola:
              task.subTrackUp(ref_FM_state.fileID.toString());
              
              Boolean[] copias = ((Boolean[]) object);
              //Cuenta el numero de copias del fragmento insertado:
              int numSuccessfulStores = 0;
                try 
                {  
                    fileFS.SetTrackID(trackID);
                    
                } catch (IOException ex) {
                   
                    System.err.println("FileManager>>Tarea "+ref_FM_state.fileID.toString()+
                                       "Error al escribir el fragmento en archivo FS. "+rice.p2p.util.MathUtils.toHex(trackID));
                }
              
              //Numero de copias insertadas:
              for (int ctr = 0; ctr < copias.length; ctr++) if (copias[ctr].booleanValue()) numSuccessfulStores++;

               
              synchronized(ref_FM_state)
              {
                  ref_FM_state.backup_tracks+=numSuccessfulStores; 
                  ref_FM_state.trackok++;
                  if(ref_FM_state.indice>ref_FM_state.totalTrack) ref_FM_state.trackbad--;
              }
              
        }

        /*Cuando hay un error en un fragmento, se almacena el indice
         *para despues intentar insertarlos. TRAS ESTE SEGUNDO INTENTO
         *se volcara el trackID al archivo FS aunque se produzca un error.
         */
        
        public void receiveException(Exception ex) {
               task.subTrackUp(ref_FM_state.fileID.toString());
              
               if(ex.toString().contains("CUBEFS:0000"))        //Fragmento ya almacenado.
               {
                    synchronized(ref_FM_state) { ref_FM_state.trackok++; }
                    try 
                    { fileFS.SetTrackID(trackID); } 
                    catch (IOException err) 
                    {
                        System.err.println("Error al escribir el fragmento en archivo FS. "+rice.p2p.util.MathUtils.toHex(trackID));
                    }            
               }
               else
               {
                   synchronized(ref_FM_state) { ref_FM_state.trackbad++; }  

                   if(ex.toString().contains("CUBEFS:0001"))    //Hash key erroneo.
                   {
                       System.err.println("Error: key hash incorrecto: "+rice.p2p.util.MathUtils.toHex(trackID)+" en el fichero fs "+ref_FM_state.fsFile);
                   }
                   else
                   {
                        //System.err.println("Error al almacenar el fragmento: "+rice.p2p.util.MathUtils.toHex(fragmento) +" "+ex.toString());     
                        ref_FM_state.conjunto.add(indice);
                   }
               }
                                
             
        }  
    }
    
    /**
     * Contiene las tareas a realizar tras la descarga de un fragmento. Funciona a modo de evento.
     * Cuando la insercion se produce, Past emplea esta clase.<br>
     * Requiere:<br>
     * -trackID.
     * -Referencia al FM_state.<br>
     * -Referencia al trackManager.<br>
     * -Referencia al gestor de tareas.<br>
     * -Referencia comando de continuacion pasado al FileManager.
     */   
    class Task_down implements Continuation
    {
       /**
        * Indentificador del fragmento.
        */
        byte trackID [];
        //Referencia al FM_state:
        Data_FileState ref_FM_state;
        //Referencia al trackManager:
        TrackManager refTrackManager;
        //Referencia del supervisor de tareas:
        Task task;
        //Referencia al comando de continuacion del FileManager:
        Continuation refCommand;
      
        /**
         * Resultado favorable.
         */
        public void receiveResult(Object object) 
        {
            task.subTrackDown(ref_FM_state.fileID.toString());
            Exception err;
            try 
            {   
                refTrackManager.WriteTrack((CubeFSTrack) object);
                synchronized(ref_FM_state) 
                { 
                    ref_FM_state.trackok++; 
                    if(ref_FM_state.indice>ref_FM_state.totalTrack) ref_FM_state.trackbad--;
                }
            
                return;
            } 
                    
            //Clave de cifrado no encontrada:
            catch (KeyCipherNotFound ex) { err = ex; }
            //Error en la clave de cifrado:
            catch(InvalidKeyException ex) { err = ex; }        
            
            ref_FM_state.setState(FileManager.STATE_TERMINATE);
            
            refCommand.receiveException(err); 
        }
        /**
         * Error.
         */
        public void receiveException(Exception exception) 
        {
            task.subTrackDown(ref_FM_state.fileID.toString());
            System.err.println("No se pudo recuperar el fragmento:" +rice.p2p.util.MathUtils.toHex(trackID) + " del archivo: " + ref_FM_state.workfile);
            synchronized(ref_FM_state) {  ref_FM_state.trackbad++; }
            
           if( exception instanceof java.lang.NullPointerException )
            {
                //ref_FM_state.setState(FileManager.STATE_TERMINATE); 
                
                return;
            }
          
            
     
        }
    }
    
    /**
     * Tareas a realizar en la fase que determina si un fragmento erroneo por time-out
     * se ha insertado de forma correcta.
     */
    class Task_Track_Exist implements Continuation
    {   
        //Para sincronizar hilos:
        BoxSyncr syncr;
        
        public void receiveResult(Object object) {
            
                syncr.aux = (Boolean) object;
                syncr.Go();    
        }

        public void receiveException(Exception exception) {
 
                syncr.aux = (Boolean) false;
                syncr.Go();           
        }
        
    }