/*
* Title			:   MetadataFile.java
* Package		:   service.cubefs
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Contiene los metadatos de un archivo en el anillo. Estos se almacenaran
*                           en el fichero .fs.
*          
*                           		
* Created on            :   08/07/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package service.cubefs;

import java.util.Calendar;
import java.util.Date;

/**
 * Contiene informacion adicional de los ficheros. Si algun datos no tiene valor
 * este estara a <B>null</B>.
 * @author victor Puertas.
 */
public class MetadataFile implements java.io.Serializable, Cloneable {
    //Nombre del autor:
    private String author = " ";
    //Fecha de creación:
    private String date_text = " ";
    //Copyright:
    private String copyright = " ";
    //Licencia:
    private String license = " ";
    //Descripcion:
    private String description = " ";

    public MetadataFile(){} 
    public MetadataFile(String descripcion, String author, Date date, String copyright, String license ) 
    {
        try { SetDescription(descripcion.substring(0,10000)); } catch (IndexOutOfBoundsException ex){ SetDescription(descripcion); }
        try { SetAuthor(author.substring(0,1000)); } catch (IndexOutOfBoundsException ex){ SetAuthor(author); }
        try { SetCopyright(copyright.substring(0,1000)); } catch (IndexOutOfBoundsException ex){ SetCopyright(copyright); }
        try { SetLicense(license.substring(0,100000)); } catch (IndexOutOfBoundsException ex){ SetLicense(license); }
        SetDate(date);
    }
    
    /**
     * Obtiene el nombre del autor.
     */
    public String GetAuthor(){ return author; }
    /**
     * Obtiene el copyright
     */
    public String GetCopyright() { return copyright; }
    /**
     * Obtiene la licencia bajo la que se distribuye el fichero.
     */
    public String GetLicense() { return license; }
    /**
     * Obtiene la fecha de creacion.
     */
    public String GetDate() { return date_text; }
    /**
     * Obtiene la descripcion del contenido.
     */
    public String GetDescription() { return description; }
    
    /**
     * Establece el autor.
     */
    public void SetAuthor(String author)
    {
        if(author.length()>0)
            this.author = author; 
    }
    
    /**
     * Establece el copyright.
     */
    public void SetCopyright(String copyright) 
    { 
        if(copyright.length()>0)
            this.copyright = copyright;      
    }
    
    /**
     * Establece la licencia.
     */
    public void SetLicense(String license) 
    {
        if(license.length()>0)
            this.license = license; 
    }
    
    /**
     * Establece la descripcion del fichero.
     */
    public void SetDescription(String description)
    {
         if(description.length()>0)
            this.description = description; 
    }
    /**
     * Establece la fecha.
     */
    public void SetDate(Date date) 
    {
        if(date == null) return;
        
        Calendar calendario = Calendar.getInstance();
        calendario.setTime(date);
        date_text = calendario.getTime().toString(); 
        try { date_text = calendario.getTime().toString().substring(0,256); } catch (IndexOutOfBoundsException ex){date_text = calendario.getTime().toString();}
    }
    
     /**
      * Permite clonar el objeto.
      */
     public Object clone()
     {
         MetadataFile obj = null;
        try 
        {
            obj = (MetadataFile) super.clone();
        } catch (CloneNotSupportedException ex) {
            //Solo en depuracion
            System.err.println("No se puede clonar el MetadataFile.");
        }
         
        obj.author = new String(this.author);
        obj.description = new String(this.description);
        obj.copyright = new String(this.copyright);
        obj.date_text = new String(this.date_text);
        obj.license = new String(this.license);
        return obj;
     }
    
}
