/*
* Title			:   CubeNodeIdFactory.java
* Package		:   cube.common.pastry
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Implementa el generador de identificadores de nodo para el anillo.
*                           		
* Created on            :   21/07/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package cube.common.pastry;

import cube.common.logger.LogZero;
import cube.common.logger.NoActiveLoggerException;
import cube.common.postman.Postman;
import cube.common.security.Cryptography;
import cube.common.util.util;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateEncodingException;
import java.util.logging.Level;
import java.util.logging.Logger;
import rice.pastry.Id;
import rice.pastry.NodeIdFactory;

/**
 * Generador de identificadores de nodo adaptado a la plataforma Cube.
 * El identificador generado es almacenado en 
 * @author victor Puertas.
 */
public class CubeNodeCertIdFactory implements NodeIdFactory {
    
    //Certificado del nodo.
    private java.security.cert.Certificate certNode;
    
    public CubeNodeCertIdFactory(java.security.cert.Certificate certNode){ this.certNode = certNode;}

    /**
     * Genera el nodeID teniendo en cuenta:<br> 
     * <B>clave publica del certificado de nodo. <br>
     * Nombre del nodo.</B><br>
     * Ya no se utiliza el random chain.
     * Importante: Si alguno de estos parametros cambia, el nodeID cambia tambien.
     * @return Identificador del nodo.
     * @return <B>NULL</B> si hay algun problema.
     */
    public Id generateNodeId() {
        
        byte public_key [];
        byte concatenado [];
        int indice;
        String nodeName = Postman.getNodeName(); 
            
        try { public_key = certNode.getEncoded(); }
            
        catch (CertificateEncodingException ex) {
            System.err.println("Error: Se ha producido un error al extraer la clave publica del certificado de nodo."); return null;
        }
        
        //reservamos el tamaño necesario:
        concatenado = new byte [public_key.length + nodeName.length()];
        
        indice = util.concatbytes(public_key, concatenado, 0, public_key.length);
        indice = util.concatbytes(nodeName.getBytes(), concatenado, indice, nodeName.getBytes().length);
        try {
            
            return Id.build(Cryptography.digestSHA1(concatenado));       
        } 
        catch (NoSuchAlgorithmException ex) 
        {
            System.err.println("Error: Se ha producido al obtener el resumen de los datos para generar el nodo.");
            return null; 
        }       
    }  
    
    /**
     * Obtiene el certificado asociado con el generador de identificadores.
     */
    public java.security.cert.Certificate getCertificate(){ return certNode; }
}
