/*
* Title			:   postman.java
* Package		:   cube.common.postman
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Cargador de servicios y configuraciones generales.
*                           		
* Created on            :   02/07/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package cube.common.postman;

import cube.common.security.Cryptography;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.util.Properties;

/**
 * Se encarga de iniciar y proporcionar la informacion global a todos los servicios
 * @author victor Puertas
 */
public class Postman {
    
/********************************************************************************************
 *      INFORMACION DE LA ESTRUCTURA DE DIRECTORIOS:
 ********************************************************************************************/
   /**
    * Directorio raiz de Cube.
    */ 
   static private String CubePlatformRootDirectory ="";
   /**
    * Directorio raiz de la plataforma. Es el direcotirio en el que se realiza la instalacion.
    */ 
   static private String AppRootDirectory ="";
   /**
    * Nombre del directorio destinado a albergar archivos temporales.
    */
   static public final String directoryTemporal ="temp";
    /**
    * Nombre del directorio destinado a albergar los archivos de configuracion.
    */
   static public final String directoryComfig ="conf";
   /**
    * Nombre del directorio destinado a albergar todo lo relacionado con la seguridad y cifrado.
    */
   static public final String directorySecurity ="security";
    /**
    * Nombre del directorio destinado a albergar los Jar servicios y librerias que 
    * forman la plataforma.
    */
   static public final String directoryLib ="lib";
    /**
    * Nombre del directorio destinado a albergar archivos de documentacion.
    */
   static public final String directoryDoc ="doc";
   
   /**
    * Nombre del fichero en el que se almacenan los certificados.
    * El archivo se almacena en /security.
    */ 
   static public final String CertKeyStoreFile = "CertStore.sec"; 
   
   /**
    * Nombre del fichero de configuracion del anillo pastry:
    */
   static public final String RING_PARAM_FILE ="ring.params";
   
   /**
    * Extension por defecto para los ficheros temporales.
    * Se almacenan en <cubeplatform>/temp
    */ 
   static public final String extensionTempFile = "tmp"; 
   
   /**
    * Parametros de configuracion de Cube platoform.
    */
   static private Properties Cube_parameter = new Properties();
   
   //Apoyo criptografico:
   static private Cryptography engine;
   static private Postman myInstance;
        
    
/********************************************************************************************
 *      SEGURIDAD:
 ********************************************************************************************/
    
   /**
    * Constructor.
    * Es necesario en se encuentre definida la propiedad cube.rootpath en el sistema
    * con el directorio de instalacion de Cube.
    * @todo Se debe consultar el nombre del nodo y el cuberoot mediante metodo getConfigParamater().
    */
   private Postman() 
   {
         engine = Cryptography.getInstance();
         CubePlatformRootDirectory = System.getProperty("cube.rootpath");
         AppRootDirectory = System.getProperty("app.install_path");
        try {
            engine.openCertStore(new File(getCubeDirectoryRoot()+File.separator+directorySecurity));
        } catch (CertificateException ex) {
            System.err.println("Error: Alguno de los certificados almacenados no se ha podido cargar.");
        } catch (NoSuchAlgorithmException ex) {
            System.err.println("Error: No se puede comprobar la integridad del almacen de claves.");
        } catch (KeyStoreException ex) {
            System.err.println("Error: Se ha producido un error general en el keyStore.");
        } catch (IOException ex) {
            System.err.println("Error: I/O.");
        }
         
          /************************************************************************************
         *  Carga de archivos de configuracion.
         *  Atencion: El conf_cubeFS deberia cargarlo el propio servicio.
         *  @todo: En el fichero cong_cubefs lo cargara el servicio.
         ************************************************************************************/
         FileInputStream archivo_conf = null;
         Properties opciones = new Properties(System.getProperties());

        try {
            
            //Propiedades de Cube Platform:
            System.err.println("Archivo de configuracion de la plataforma: "+Postman.getCubeDirectoryRoot()+File.separator+Postman.directoryComfig+File.separator+"conf_cube");
            archivo_conf = new FileInputStream(Postman.getCubeDirectoryRoot()+File.separator+Postman.directoryComfig+File.separator+"conf_cube");
            Cube_parameter.load(archivo_conf);
            archivo_conf.close();
            
            String chain = Cube_parameter.getProperty("cube.chain");
            if(chain==null || chain.length()<20)
            {
                Cube_parameter.setProperty("cube.chain",rice.p2p.util.MathUtils.toHex(engine.getRandom(20)));
                FileOutputStream archivo = new FileOutputStream(Postman.getCubeDirectoryRoot()+File.separator+Postman.directoryComfig+File.separator+"conf_cube");
                Cube_parameter.store(archivo,"Archivo de configuracion de Cube Platform.");
            }
            

            
            archivo_conf = new FileInputStream(Postman.getCubeDirectoryRoot()+File.separator+Postman.directoryComfig+File.separator+"conf_cubeFS");
            opciones.load(archivo_conf);
            archivo_conf.close();
            System.setProperties(opciones);
            
        } catch (FileNotFoundException ex) {
           System.err.println("Error: Archivo de configuracion no encontrado.");
       
        }catch (IOException ex) {
            System.err.println("Error: se ha producido un error al cargar el archivo de configuracion.");
            
        }
    }
   /**
    * Obtiene la instancia del postman mediante el patron Singleton.
    * La primera vez que se obtenga la instancia hay que asegurarse que este definida
    * la propiedad <b>cube.rootpath</b> con el directorio de instanlacion de cube.
    */
   static public Postman getIntance()
   { 
       if(myInstance == null) { myInstance = new Postman();} 
       return myInstance;
   } 
    
   /**
    * Obtiene el directorio raiz de la plataforma.
    */
   static public String getCubeDirectoryRoot() { return CubePlatformRootDirectory; }
   
   /**
    * Obtiene el directorio en el que se instalaron todos los servicios de Cube.
    */
   static public String getAppRoot(){return AppRootDirectory;}
   
   /**
    * Devuelve el nombre del nodo. Se emplea para generar el identificador.
    * Si no esta definido en el archivo de configuracion devuelve: cadena vacia.
    */
   static public String getNodeName()
   { 
       String salida = getConfigParamater("cube.nodeName"); 
       if(salida==null) return "";
       return salida;
   }
   /**
    * Permite consultar los paramtros de configuracion de la plataforma en general.
    * @param param Parametro a consultar.
    */
   static public String getConfigParamater(String param){ return Cube_parameter.getProperty(param); }
    
    /**
     * Obtiene una referencia del Cube Cryptography Engine empleado en Cube.
     * @return Cube Cryptography Engine
     */
    static public Cryptography GetRefCryptography() { return engine; }
    
}
