/*
* Title			:   Cryptography.java
* Package		:   cube.common.security
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Contiene las funcionalidades de criptografia de la plataforma
*                           Cube.             
*                           		
* Created on            :   28/05/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package cube.common.security;

import cube.common.logger.LogZero;
import cube.common.logger.NoActiveLoggerException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Enumeration;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.crypto.*;
import javax.crypto.spec.SecretKeySpec;



/**
 * Implementa las opciones de cifrado de Cube. Permite:
 * -Generar claves de cifrado.
 * -Almacenar y recuperar las claves a traves de un alias.
 * -Obtener resumen de datos.
 * -Generar numeros aleatorios.
 * -Trabajar con certificados.
 * 
 * Las claves publica-privada es necesaria pues se emplea para autentificar
 * nodos, generar identificadores de nodo en Pastry y crear identificadores de archivo.
 * 
 * 
 * Algoritmos empleados:
 * <B>Cifrado de datos: AES de 128 bits, 
 * Infraestructura clave publica: RSA - 2048,
 * Resumen de contenido: SHA-1</B>.
 * @Deprecated.
 */
public class Cryptography_ {
    
    //Nombre del algoritmo de cifrado de datos:
    static private final String algorithm_name = "AES";
    //Tamaño clave empleada en el algoritmo de cifrado de datos:
    static private final int longKey = 128;  
    //Generador de claves de cifrado de datos (AES):
    static private KeyGenerator keygen;
    
    //Nombre del algoritmo empleado en infraestructura de clave publica:
    static private final String algorithm_PKI_name = "RSA";
    
    /**
     * Indica el alias asignado a certificado de nodo.
     */
    static public final String alias_nodeCert = "Cube_cert_x509";
    /**
     * Indica el alias del la autoridad certificadora del anillo.
     */
    static public final String alias_CA = "Cube_CA";
    
    //Tipo de almacen del KeyStore:
    static private final String keystore_sort_database = "JCEKS";
    //Password empleado para almacenar las claves:
    static private final char [] keystore_pass_database = {'a','8','4','j','d','7','1','l','d','1'};
    //Keystore: Almacenador de claves.
    static private KeyStore KeyDB;
    //Archivo de almacen de claves:
    static private File keydatabaseFile;
   
    //Nombre del algoritmo resumen empleado:
    static private final String digest_Algorith_name = "SHA1";
    //Clase para calcular el resumend de los datos:
    static private MessageDigest digest;
    
    //Generador de numeros aleatorios:
    static private SecureRandom random;
    //Nombre del algoritmo para generar numeros aleatorios:
    static private final String random_algorithm_name = "SHA1PRNG";
    
    //Determina si esta activado el sistema de cifrado. Si AES no esta disponible esta desactivado.
    static private boolean activeEngine = false;
    //Tamaño del buffer de datos para operar con Input/Ouput Stream:
    static private int long_buffer_data = 8;
    
    //Logger de la clase:
    static private Logger mylog;
    //Nombre del logger:
    static private final String logger_name = "common.cube_cryptography_engine";
    
    //Constantes para obtener el nombre de algoritmo empleado en cada una de las actividades:
    /**
     * Obtiene el nombre del algoritmo empleado para cifrar los datos.
     */
    static public final int algorithm_cipher = 0;
    /**
     * Obtiene el nombre del algoritmo empleado para generar números aleatorios.
     */
    static public final int algorithm_ramdom = 1;
    /**
     * Obtiene el nombre del algoritmo empleado para obtener resúmenes de datos.
     */
    static public final int algorithm_digest = 2;
  
    static private Cryptography_ instance = null;
      
    
    /************************************************
     * Crea una instancia del Cube Cryptography Engine
     * mediante el patron Singleton.
     * @param kSfile Archivo que se empleara como almacen de claves (KeyStore). Se debe pasar el path absoluto.
     * Si el archivo no existe se generara un nuevo almacen.
     * @throws java.security.NoSuchAlgorithmException El proveedor criptografico no posee la implementacion del algoritmo de cifrado.
     * Se usa el proveedor por defecto de Sun: SunJCE.
     * @throws java.security.KeyStoreException Se ha producido un error durante la obtencion de la instancia del KeyStore.
     * @throws java.security.cert.CertificateException Error en la apertura de los certificados asociados a la clave.
     * @throws java.io.IOException Error de entrada/salida durante la carga del archivo almacen de claves.
     ************************************************/
    synchronized static public Cryptography_ createInstance(String kSfile) throws NoSuchAlgorithmException, KeyStoreException, CertificateException, IOException
    {
        instance = new Cryptography_(kSfile);
        return instance;
    }
    
    /************************************************
     * Crea una instancia del Cube Cryptography Engine
     * mediante el patron Singleton.
     * @param kSfile Archivo que se empleara como almacen de claves (KeyStore). Se debe pasar el path absoluto.
     * Si el archivo no existe se generara un nuevo almacen.
     * @param privateFileKey Clave privada del certificado.
     * @param certFile Archivo con el certificado con formato UIT-T x.509 en binario.
     * @throws java.security.NoSuchAlgorithmException El proveedor criptografico no posee la implementacion del algoritmo de cifrado.
     * Se usa el proveedor por defecto de Sun: SunJCE.
     * @throws java.security.KeyStoreException Se ha producido un error durante la obtencion de la instancia del KeyStore.
     * @throws java.security.cert.CertificateException Error en la apertura de los certificados asociados a la clave.
     * @throws java.io.IOException Error de entrada/salida durante la carga del archivo almacen de claves.
     ************************************************/
    synchronized static public Cryptography_ createInstance(String kSfile, String privateFileKey, String certFile) throws NoSuchAlgorithmException, KeyStoreException, CertificateException, IOException
    {
        instance = new Cryptography_(kSfile, privateFileKey, certFile);
        return instance;
    }
     
    synchronized static public Cryptography_ getInstance() {return instance;}
    
    /**
     * Constructor que inicializa los objetos del sistema de cifrado:
     * <BR>Generación de claves
     * <BR>Almacen de claves
     * <BR>Cifrado de datos
     * <BR>Generar resumenes de datos
     * <BR>Generador de numeros aleatorios
     * <BR>Administración de certificados
     * @param file Archivo que se empleara como almacen de claves (KeyStore). Se debe pasar el path absoluto.
     * Si el archivo no existe se generara un nuevo almacen.
     * @throws java.security.NoSuchAlgorithmException El proveedor criptografico no posee la implementacion del algoritmo de cifrado.
     * Se usa el proveedor por defecto de Sun: SunJCE.
     * @throws java.security.KeyStoreException Se ha producido un error durante la obtencion de la instancia del KeyStore.
     * @throws java.security.cert.CertificateException Error en la apertura de los certificados asociados a la clave.
     * @throws java.io.IOException Error de entrada/salida durante la carga del archivo almacen de claves.
     */
     private Cryptography_(String file) throws NoSuchAlgorithmException, KeyStoreException, CertificateException, IOException 
     {
         File kSfile = new File(file);
        //Si el Cube Cryptography Engine esta activo salimos. De lo contrario procedemos a la carga.
        if(GetActiveEngine()){return;}
        try {mylog = LogZero.CreateSubLogger(logger_name);} catch (NoActiveLoggerException ex) {}
          
        //Generador de claves:
        WriteLog(Level.INFO,"Iniciando: Generador de claves...\n");
        try {
            
            //Inicio del generador de claves:
            keygen = KeyGenerator.getInstance(algorithm_name);
            //Longitud de clave:
            keygen.init(longKey);
            
        } catch (NoSuchAlgorithmException ex) {
            SetActiveEngine(false);
            WriteLog(Level.SEVERE,"KEYGENERATOR: El proveedor de cifrado no proporciona el algoritmo " + keygen.getAlgorithm() +".\n");
        
            throw new NoSuchAlgorithmException();
        }
        
        //Generador de numeros aleatorios: 
        WriteLog(Level.INFO,"Iniciando: Generador de números aleatórios con algoritmo: " + random_algorithm_name + "\n");
        try
        {
            random = SecureRandom.getInstance(random_algorithm_name);
        }
        catch (NoSuchAlgorithmException ex)
        {
            WriteLog(Level.SEVERE,"RandomNumber: Los proveedores criptograficos instalados no propocionan el algoritmo algoritmo: " + random_algorithm_name + ".\n");
            SetActiveEngine(false);
            throw new NoSuchAlgorithmException("RandomNumber");
        }
        
 
        
        //Calculador de resumen de contenido. Algoritmo por defecto: SHA-1.
        WriteLog(Level.INFO,"Iniciando: Generador de resumen de contenido con algoritmo: " + digest_Algorith_name + "\n");
        try {
            digest = MessageDigest.getInstance(digest_Algorith_name);
        } catch (NoSuchAlgorithmException ex) {
            WriteLog(Level.WARNING,"Digest: El proveedor: " + digest.getProvider() + " no soporta el algoritmo " + digest.getAlgorithm() + ".\n");
            throw new NoSuchAlgorithmException ("SHA-1");
        }
               
        //Almacen de claves:
        WriteLog(Level.INFO,"Iniciando: Almacen de claves (KEYSTORE)...\n");
        
        keydatabaseFile = kSfile;
        FileInputStream keydatabase = null;
        
        try {
            keydatabase = new FileInputStream(keydatabaseFile);
        } catch (FileNotFoundException ex) {
           WriteLog(Level.SEVERE,"kEYSTORE: Creando el archivo de claves: " + kSfile.getAbsolutePath() +".\n");
        }
        
        try {
            KeyDB = KeyStore.getInstance(keystore_sort_database);
        } catch (KeyStoreException ex) 
        { WriteLog(Level.SEVERE,"KEYSTORE: Tipo "+ keystore_sort_database + " no se encuentra disponible.\n");
          SetActiveEngine(false); new KeyStoreException(keystore_sort_database+" no activo.\n");}
        
        try {        
            KeyDB.load(keydatabase,keystore_pass_database); 
            WriteLog(Level.INFO,"KEYSTORE: Fichero de claves cargado.\n");
        } catch (CertificateException ex) {
            WriteLog(Level.SEVERE,"KEYSTORE: No se pueden cargar los certificados asociados a las claves.\n");
            throw new CertificateException("KEYSTORE");
        } catch (NoSuchAlgorithmException ex) {
             WriteLog(Level.SEVERE,"KEYSTORE: No se encuentra el algoritmo de cifrado empleado por "+ keystore_sort_database + "\n");
             throw new NoSuchAlgorithmException("KEYSTORE");
        } catch (IOException ex) {
             WriteLog(Level.SEVERE,"KEYSTORE: Error I/O en el archivo de claves.\n");
             throw new IOException("KEYSTORE");
        }
        try {
            if(keydatabase!=null){keydatabase.close();} else {keydatabaseFile.createNewFile();}
        } catch (IOException ex) {
             WriteLog(Level.SEVERE,"KEYSTORE: Error durante el cierre del archivo de claves de entrada.\n");
             throw new IOException("KEYSTORE: cerrando el stream de entrada.");
        }
                
        //Comprobamos si el certificado del nodo con formato U-ITT X.509 se encuentra:
        try
        {
            if(!KeyDB.isKeyEntry(alias_nodeCert))
            {
                WriteLog(Level.SEVERE,"El certificado del nodo con formato U-ITT X.509 no se encuentra disponible en el KeyStore.\n");
                SetActiveEngine(false);
                return;
            }
        }
        catch (KeyStoreException ex)
        {
             WriteLog(Level.SEVERE,"Se ha obtenido un error al durante la consulta del certificado del nodo en el KeyStore.\n");
             SetActiveEngine(false);
             throw new KeyStoreException("KEYSTORE: Cube_cert_X509");      
        }
        
        
        SetActiveEngine(true);
        WriteLog(Level.INFO,"Iniciado: Cube Cryptography Engine.\n");
    }
    
    /**
     * Constructor que inicializa los objetos del sistema de cifrado:
     * Esta llamada se emplea para inicializar el certificado de nodo, que permitirá obtener 
     * los identificadores necesarios para el funcionamiento de la plataforma.
     * @param file Archivo que se empleara como almacen de claves (KeyStore). Se debe pasar el path absoluto.
     * Si el archivo no existe se generara un nuevo almacen.
     * @param privateFileKey Clave privada del certificado.
     * @param certFile Archivo con el certificado con formato UIT-T x.509 en binario.
     * @throws java.security.NoSuchAlgorithmException El proveedor criptografico no posee la implementacion del algoritmo de cifrado.
     * Se usa el proveedor por defecto de Sun: SunJCE.
     * @throws java.security.KeyStoreException Se ha producido un error durante la obtencion de la instancia del KeyStore.
     * @throws java.security.cert.CertificateException Error en la apertura de los certificados asociados a la clave.
     * @throws java.io.IOException Error de entrada/salida durante la carga del archivo almacen de claves.
     */
     private Cryptography_ (String file, String privateFileKey, String certFile) throws NoSuchAlgorithmException, KeyStoreException, CertificateException, IOException
     {
         File kSfile = new File(file);
         //Si el Cube Cryptography Engine esta activo salimos. De lo contrario procedemos a la carga.
        if(GetActiveEngine()){return;}
        try {mylog = LogZero.CreateSubLogger(logger_name);} catch (NoActiveLoggerException ex) {}
          
        //Generador de claves:
        WriteLog(Level.INFO,"Iniciando: Generador de claves...\n");
        try {
            
            //Inicio del generador de claves:
            keygen = KeyGenerator.getInstance(algorithm_name);
            //Longitud de clave:
            keygen.init(longKey);
            
        } catch (NoSuchAlgorithmException ex) {
            SetActiveEngine(false);
            WriteLog(Level.SEVERE,"KEYGENERATOR: El proveedor de cifrado no proporciona el algoritmo " + keygen.getAlgorithm() +".\n");
        
            throw new NoSuchAlgorithmException();
        }
        
        //Generador de numeros aleatorios: 
        WriteLog(Level.INFO,"Iniciando: Generador de números aleatórios con algoritmo: " + random_algorithm_name + ".\n");
        try
        {
            random = SecureRandom.getInstance(random_algorithm_name);
        }
        catch (NoSuchAlgorithmException ex)
        {
            WriteLog(Level.SEVERE,"RandomNumber: Los proveedores criptograficos instalados en Java no propocionan el algoritmo algoritmo: " + random_algorithm_name + "\n");
            SetActiveEngine(false);
            throw new NoSuchAlgorithmException("RandomNumber");
        }
        
         
        //Calculador de resumen de contenido. Algoritmo por defecto: SHA-256.
        WriteLog(Level.INFO,"Iniciando: Generador de resumen de contenido con algoritmo: " + digest_Algorith_name + "\n");
        try {
            digest = MessageDigest.getInstance(digest_Algorith_name);
        } catch (NoSuchAlgorithmException ex) {
            WriteLog(Level.WARNING,"Digest: El proveedor: " + digest.getProvider() + " no soporta el algoritmo " + digest.getAlgorithm() + ".\n");
            throw new NoSuchAlgorithmException (digest_Algorith_name);
        }
               
        //Almacen de claves:
        WriteLog(Level.INFO,"Iniciando: Almacen de claves (KEYSTORE)...\n");
        
        keydatabaseFile = kSfile;
        FileInputStream keydatabase = null;
        
        try {
            keydatabase = new FileInputStream(keydatabaseFile);
        } catch (FileNotFoundException ex) {
           WriteLog(Level.SEVERE,"kEYSTORE: Creando el archivo de claves: " + kSfile.getAbsolutePath() +".\n");
        }
        
        try {
            KeyDB = KeyStore.getInstance(keystore_sort_database);
        } catch (KeyStoreException ex) 
        { WriteLog(Level.SEVERE,"KEYSTORE: Tipo "+ keystore_sort_database + " no se encuentra disponible.\n");
          SetActiveEngine(false); new KeyStoreException(keystore_sort_database+" no activo.\n");}
        
        try {        
            KeyDB.load(keydatabase,keystore_pass_database); 
            WriteLog(Level.INFO,"KEYSTORE: Fichero de claves cargado.\n");
        } catch (CertificateException ex) {
            WriteLog(Level.SEVERE,"KEYSTORE: No se pueden cargar los certificados asociados a las claves.\n");
            throw new CertificateException("KEYSTORE");
        } catch (NoSuchAlgorithmException ex) {
             WriteLog(Level.SEVERE,"KEYSTORE: No se encuentra el algoritmo de cifrado empleado por "+ keystore_sort_database + ".\n");
             throw new NoSuchAlgorithmException("KEYSTORE");
        } catch (IOException ex) {
             WriteLog(Level.SEVERE,"KEYSTORE: Error I/O en el archivo de claves.\n");
             throw new IOException("KEYSTORE");
        }
        try {
            if(keydatabase!=null){keydatabase.close();} else {keydatabaseFile.createNewFile();}
        } catch (IOException ex) {
             WriteLog(Level.SEVERE,"KEYSTORE: Error durante el cierre del archivo de claves de entrada.\n");
             throw new IOException("KEYSTORE: cerrando el stream de entrada.");
        }
                
        //Comprobamos si el certificado del nodo con formato U-ITT X.509 se encuentra:
        SetActiveEngine(true);
                if(!InsertCert(alias_nodeCert, privateFileKey, certFile))
                {
                    WriteLog(Level.SEVERE,"Autenticación del nodo: Error al insertar el certificado con alias: " + alias_nodeCert + " en le keystore.\n");
                    SetActiveEngine(false);
                    return;
                }       
        WriteLog(Level.INFO,"Iniciado: Cube Cryptography Engine.\n");
    }
        
    /**
     * Cierra el Cube Cryptography Engine: Vuelca a archivo las claves situadas en memoria para poder
     * obtenerlas posteriormente. El archivo sera el especificado como parámetro en el InitSystem().
     * <B>Se debe llamar cuando se cierre la plataforma Cube.</B>
     * @throws java.io.FileNotFoundException No se encuentra el archivo designado para almacenar las claves.
     * @throws java.security.KeyStoreException Error general en el KeyStore.
     */
     synchronized static public void CloseInstance() throws FileNotFoundException, KeyStoreException
     {
        if(!GetActiveEngine()){return;}
        WriteLog(Level.INFO,"Cerrando el Cube Cryptography Engine...\n");
        
        FileOutputStream archivosalida = null;
        try {
            archivosalida = new FileOutputStream(keydatabaseFile);
            KeyDB.store(archivosalida,keystore_pass_database);
            WriteLog(Level.INFO,"KEYSTORE: Archivo de claves cerrado con exito.\n");
            SetActiveEngine(false);
            System.gc();
            return;
        } catch (FileNotFoundException ex) {
            WriteLog(Level.SEVERE,"KEYSTORE: No se encuentra el archivo en el que volcar las claves del KeyStore.\n");
            throw new FileNotFoundException ();
        }catch (KeyStoreException ex) {
            WriteLog(Level.SEVERE,"KEYSTORE: Se ha producido un error inesperado al volcar el KeyStore en el archivo.\n");
            throw new KeyStoreException ("KEYSTORE: Cerrando el Cube Cryptography Engine.");
        } catch (NoSuchAlgorithmException ex) {
        } catch (CertificateException ex) {
        } catch (IOException ex) {}
        
        instance = null;
        WriteLog(Level.SEVERE,"KEYSTORE: Se ha producido un error grabe al volcar el KeyStore a archivo durante el cierre del Cube Cryptography Engine.\n");       
     }
    
    /**
     * Inserta un certificado en el keystore. Los alias <B>Cube_cert_x509</B> y <B>Cube_CA</B> estan
     * reservados para determinar el <B>certificado del nodo</B> y la <B>autoridad certificadora
     * del anillo</B>. <B>Este metodo solo se debe llamar cuando esta activo el Cube Cryptograpy Engine</B>.
     * <PRE>Formato de la clave privada:</PRE> PKCS8 sin password. Puede emplear la utilidad openssl de
     * Linux para convertir a este formato de clave:
     * <CODE> openssl pkcs8 -topk8 -outform DER -in private.key -out privada_pkcs8.key -nocrypt</CODE>
     * 
     * <PRE>Formato del certificado:</PRE> UIT-T x.509.
     * @param alias Alias asociado al certificado. No pude ser <B>Cube_cert_x509</B> ni <B>Cube_CA</B>.
     * @param privateFileKey Clave privada asociada al certificado.
     * @param certFile Certificado con formato UIT-T x.509 en formato binario.
     * @return <I>true</I> El certificado se almaceno con exito, <I>false</I> de lo contrario. Ha ocurrido un error o el 
     * Cube Cryptography Engine no esta activo.
     */
    synchronized static public boolean InsertCert(String alias, String privateFileKey, String certFile)
     {
        if(!GetActiveEngine())
        {
           WriteLog(Level.SEVERE,"Certificados: No se puede insertar el certificado. Cube Cryptography Engine no está activo.\n");
           return false;
        }
        //Lectura de la clave privada:
        File f = new File(privateFileKey);
        byte[] encodedKey = new byte[(int) f.length()];
        FileInputStream fis = null;
        PrivateKey privKey;
        
        try {
            fis = new FileInputStream(privateFileKey);
           
        } catch (FileNotFoundException ex) {
            WriteLog(Level.SEVERE,"Certificados: El archivo con la clave privada no se encuentra.\n");
            return false;
        }
        try {
            fis.read(encodedKey);
        } catch (IOException ex) {
            WriteLog(Level.SEVERE,"Certificados: Error I/O durante la lectura del buffer que contiene la clave privada.\n");
            return false;
        }
        try {
            fis.close();
        } catch (IOException ex) {
            WriteLog(Level.SEVERE,"Certificados: Error I/O al cerrar el archivo que contiene la clave privada.\n");
            return false;
        }
        
        //Crear el objeto con la clave privada:
        PKCS8EncodedKeySpec kspec = new PKCS8EncodedKeySpec(encodedKey);
        KeyFactory kfac = null;
        try {
            kfac = KeyFactory.getInstance(algorithm_PKI_name);
        } catch (NoSuchAlgorithmException ex) {
            WriteLog(Level.SEVERE,"Certificados: No hay proveedor criptografico que proporcione" + algorithm_PKI_name + ".\n");
            return false;
        }
        try {
                privKey = kfac.generatePrivate(kspec);
        } catch (InvalidKeySpecException ex) {
            WriteLog(Level.SEVERE,"Certificados: la clave privada proporcionada no es valida. Debe estar en formato PKCS8 sin password.\n");
            return false;
        }
        
        //Lectura del archivo con el certificado:
        InputStream inStream;
        try {
            inStream = new FileInputStream(certFile);
        } catch (FileNotFoundException ex) {
            WriteLog(Level.SEVERE,"Certificados: No se encuentra el archivo con el certificado.\n");
            return false;
        }
        CertificateFactory cf;
        try {
            cf = CertificateFactory.getInstance("X.509");
        } catch (CertificateException ex) {
            WriteLog(Level.SEVERE,"Certificados: Se ha producido un error al cargar el certificado. Puede que el proveedor de servicios criptograficos\nno lo soporte\n");
            return false;
        }
        X509Certificate cert;
        try {
            cert = (X509Certificate) cf.generateCertificate(inStream);
        } catch (CertificateException ex) {
            WriteLog(Level.SEVERE,"Certificados: Se ha producido un error al crear el certificado. Puede que este corrupto o que el proveedor de servicios criptograficos\nno lo soporte\n");
            return false;
        }
        try {

            inStream.close();
        } catch (IOException ex) {
            WriteLog(Level.SEVERE,"Certificados: Se ha producido un error al cerrar el archivo con el certificado.\n");
            return false;
        }
        
        java.security.cert.Certificate[] certChain = new java.security.cert.Certificate[1];
        certChain[0]=cert;
        
        try {
            
            //Almacenamos el certificado en el keystore:
            
            KeyDB.setKeyEntry(alias, privKey, keystore_pass_database,certChain); 
        } catch (KeyStoreException ex) {
            WriteLog(Level.SEVERE,"Certificados: Error durante el almacenamiento del certificado con alias: "+ alias +".\n");
            return false;
        } 
       return true;
         
     }
     
    /**
     * Genera un numero aleatorio de 10 bytes.
     * @return Numero aleatorio o <B>null</B> si el Cube Cryptography Engine no esta activo.
     */
    synchronized public byte [] GenerateRandom()
     {
         if(!GetActiveEngine())
        {
           WriteLog(Level.SEVERE,"No se puede generar el numero aleatorio. Cube Cryptography Engine no esta activo.\n");
           return null;
        }
         
         byte [] numero = new byte [10];
         random.nextBytes(numero);
         return numero;
     }
    
    /**
     * Obtiene un certificado asociado con un alias.
     * @return El certificado o <B>null</B> en caso no existir el certificado, error en el keystore o 
     * introducir un indice no valido.
     * @param alias Alias de la entrada en el keystore.
     * @param number Indice en la cadena de certificados asociados a la entrada del keystore.
     */
    synchronized public Certificate GetCertificate(String alias, int number)
     {
         if(!GetActiveEngine())
        {
           WriteLog(Level.SEVERE,"No se puede obtener el certificado numero " + number + " asociado al alias: " + alias + ".\n");
           return null;
        }
        java.security.cert.Certificate[] certChain;
        try {
            certChain = KeyDB.getCertificateChain(alias); 
            return certChain[number];
        } catch (KeyStoreException ex) {
            WriteLog(Level.SEVERE,"Se ha producido un error en el keystore al obtener los certificados del alias: " + alias + ".\n");
            return null;
        } catch (ArrayIndexOutOfBoundsException ex){
            WriteLog(Level.INFO,"El certificado numero " + number + " no existe en la cedena de certificados asociados a " + alias + ".\n");
            return null;
        }
     }
     
    /**
     * Inserta un Certificado Verdadero, un CA, que permitirá validar las firmas de los certificados de nodo.
     * Se puede asociar cualquier alias, pero hay que tener en cuenta que para el acceso a los anillos privados
     * es necesario crear la entrada en con alias <B>Cube_CA</B> que determina la Autoridad Certificadora del 
     * anillo y que todos los miembros del anillo comparten. Ademas el certificado de nodo debe estar firmado
     * por dicha CA. Para el acceso a anillos públicos nos es necesario establecer el CA, simplemente hay que
     * disponer de un certificado que puede estar autofirmado.
     * @param alias Alias asignado al cetificado.
     * @param cert Certificado Verdadero, proporcionado por la Autoridad Certificadora.
     * @return <B>true</B> el certificado se insertó correctamente. <B>false</B> Ocurrió un error.
     * Es aconsejable revisar el log general de la plataforma.
     */
     synchronized public boolean InsertCA(String alias,Certificate cert)
     {
        try {
            KeyDB.setCertificateEntry(alias, cert);
            return true;
        } catch (KeyStoreException ex) {
            WriteLog(Level.SEVERE,"Se ha producido un error al almacenar el certificado del CA con alias: "+ alias + " en el keystore.\n");
           
        }
        return false;
     }
     
    /**
     * Obtiene el nombre del algoritmo empleado en cada una de las partes del Cube Cryptography Engine:
     * cifrado de datos, generacion de numeros aleatorios y resumen de contenido.
     * Para ello se hace uso de las siguientes constantes:
     * 
     * <BR>algorithm_cipher = 0;
     * <BR>algorithm_ramdom = 1;
     * <BR>algorithm_digest = 2;
     * <BR>
     * @param tipo Determina la informacion que se desea obtener. Emplear las constantes descritas 
     * anteriormente.
     * @return Cadena que contiene el nombre del algoritmo.
     */
     public String GetAlgorith(int tipo)
     {
         if(!GetActiveEngine()) return null;
         
         switch(tipo)
         {
             case 0:        //Algoritmo empleado para cifrar datos:
                  return keygen.getAlgorithm();
                  
             case 1:        //Algoritmo empleado para generar numeros aleatorios:
                  return random.getAlgorithm();
                  
             case 2:        //Algoritmo empleado para obtener resumenes.
                 return digest.getAlgorithm();                            
         }
         
         return null;
     }
    /**
     * Almacena una nueva clave simétrica en el KeyStore identificada por un alias. Funcionamiento:
     * Si la entrada con el alias existe, sera sustituida.
     * @param alias Alias para referirse a la clave.
     * @param clave Clave a almacenar.
     * @return TRUE: Clave insertada.<BR>
     * FALSE: Error al insertar la clave. Causa: Cube Cryptography engine no iniciado o el KeyStore
     * ha producido un error general.
     */
    synchronized public boolean NewEntryKeyStore(String alias, KeyStore.SecretKeyEntry clave)
    {
        if(alias == alias_nodeCert || alias == alias_CA)
        {
            WriteLog(Level.INFO,"KEYSTORE: El alias "+ alias + " está reservado para Cube. No se puede emplear.\n");
            return false;
        }
        
        if(GetActiveEngine())
        {
            try {
                KeyDB.setEntry(alias, clave, new KeyStore.PasswordProtection(keystore_pass_database)); 
                WriteLog(Level.INFO,"KEYSTORE: Clave con alias: "+ alias +" añadida.\n");
                return true;
            } catch (KeyStoreException ex) 
                { WriteLog(Level.INFO,"KEYSTORE: Ha ocurrido un error al añadir la clave con alias: "+ alias +".\n"); } 
        }   
        return false;
    }
    
    /**
     * Almacena una nueva clave simétrica en el KeyStore identificada por un alias. Funcionamiento:
     * Si la entrada con el alias existe, sera sustituida.
     * @param alias Alias para referirse a la clave.
     * @param clave Clave a almacenar.
     * @return TRUE: Clave insertada.<BR>
     * FALSE: Error al insertar la clave. Causa: Cube Cryptography engine no iniciado o el KeyStore
     * ha producido un error general.
     */
    synchronized public boolean NewEntryKeyStore(String alias, SecretKey clave)
    {
         if(alias == alias_nodeCert || alias == alias_CA)
        {
            WriteLog(Level.INFO,"KEYSTORE: El alias "+ alias + " está reservado para Cube. No se puede emplear.\n");
            return false;
        }
         
        if(GetActiveEngine())
        {
            try {
                KeyDB.setEntry(alias, new KeyStore.SecretKeyEntry(clave), new KeyStore.PasswordProtection(keystore_pass_database)); 
                WriteLog(Level.INFO,"KEYSTORE: Clave con alias: "+ alias +" añadida.\n");
                return true;
            } catch (KeyStoreException ex) 
                { WriteLog(Level.INFO,"KEYSTORE: Ha ocurrido un error al añadir la clave con alias: "+ alias +".\n"); } 
        }   
        return false;
    }
       
    /**
     * Elimina la entrada refenrenciada por el alias del KeyStore.
     * @param alias Referencia la clave a borrar.
     * @return TRUE: Clave borrada.<BR>
     * FALSE: La clave con el alias no se encuentra o se ha producido un error general.
     */
    synchronized public boolean DelEntryKeyStore(String alias)
    {
         if(alias == alias_nodeCert || alias == alias_CA)
        {
            WriteLog(Level.INFO,"KEYSTORE: El alias "+ alias + " está reservado para Cube. No se puede emplear.\n");
            return false;
        }
         
        if(GetActiveEngine())
        {
            try {
                KeyDB.deleteEntry(alias);
                WriteLog(Level.INFO,"KEYSTORE: Clave con alias: "+ alias +" borrada.\n");
                return true;
            } catch (KeyStoreException ex) {}
        }
        WriteLog(Level.INFO,"KEYSTORE: Ha ocurrido un error al borrar la clave con alias: " + alias + ".\n");
        return false;
    }
    
    /**
     * Consulta el KeyStore y busca la clave asociada al alias pasado como argumento.
     * <B>En caso de querer usarla en CipherBlock() se debe convertir a SecretKeySpec.</B>
     * @param alias Alias asignado a la clave almacenada.
     * @return Clave asociada al alias.<BR>NULL si el Cube Cryptography Engine no esta activo o ha ocurrido un error. Consultar
     * el archchivo .log para precisar mas.
     */
    synchronized public Key QueryKey(String alias)
    {
        if(!GetActiveEngine()) return null;
        try {
            return KeyDB.getKey(alias, keystore_pass_database);
        } catch (UnrecoverableKeyException ex) {
            WriteLog(Level.INFO,"KEYSTORE: No se puede recuperar la clave con alias "+ alias +".\n");
        } catch (KeyStoreException ex) {
            WriteLog(Level.INFO,"KEYSTORE: Ha ocurrido un error al añadir la clave con alias: "+ alias +".\n");
        } catch (NoSuchAlgorithmException ex) {
            WriteLog(Level.SEVERE,"KEYSTORE: No se encuentra el algoritmo de cifrado empleado por "+ keystore_sort_database + "\n");      
        }
        return null;
    }
    
    /**
     * Consulta el KeyStore y busca la clave asociada al alias pasado como argumento.
     * <B>Se puede usar direcmamente en  CipherBlock() y UnCipherBlock().</B>
     * @param alias Alias asignado a la clave almacenada
     * @return Clave asociada al alias.<BR>NULL si el Cube Cryptography Engine no esta activo o ha ocurrido un error. Consultar
     * el archchivo .log para precisar mas.
     */
    
     synchronized public SecretKeySpec QuerySecretKey_keySpec (String alias)
     {
         if(!GetActiveEngine()) return null;
         Key temp_key=QueryKey(alias);
         if ( temp_key==null) return null;
         return new SecretKeySpec(temp_key.getEncoded(),GetAlgorith(algorithm_cipher));
     }
    
    
    /**
     * Devuelve el número de claves que hay en el KeyStore.
     * @return Numero de claves almacenadas.
     */
    synchronized public int LongKeyStore()
    {
        if(!GetActiveEngine()) { return -1; }
        try {
            return KeyDB.size();
        } catch (KeyStoreException ex) {}
        
         return -1;
    }
    
    /**
     * Obtiene la lista de Alias alamacenados en el KeyStore.
     * @return Enumeración de los alias.
     * @throws java.security.KeyStoreException Se ha producido un error en el KeyStore durante el proceso de consulta de los alias.
     */
    synchronized public Enumeration<String> GetKeyStoreAliasList() throws KeyStoreException
    {
        if(!GetActiveEngine()) {return null;}
        try {
            return KeyDB.aliases();
        } catch (KeyStoreException ex) {
            WriteLog(Level.SEVERE,"KEYSTORE: Error al obtener la lista de alias.\n");
            throw new KeyStoreException("KEYSTORE: ALIAS.");
        }
    }
    
    /**
     * Genera una clave secreta para el algoritmo simétrico que se esta empleando en el cifrado.
     * Para ello es necesario que Cube Cryptography Engine este activo.
     */
    synchronized public SecretKey GenerateSecretKey() 
    {
       if(GetActiveEngine()){ WriteLog(Level.FINE,"Clave " + algorithm_name + " generada.\n"); return keygen.generateKey();}
       else{ WriteLog(Level.INFO,"No se puede generar la clave de cifrado: Cube Cryptography Engine no está activo.\n"); return null; }
    }
   
    /**
     * Cifra un bloque de datos almacenado en una archivo. Es usado para encriptar los 
     * fragmentos de archivo.
     * @return TRUE: Bloque de datos cifrado con exito.<BR>
     * FALSE: El bloque de datos no se ha cifrado. Causa: Cube Cryptography Engine no esta activo.
     * @throws java.io.IOException Error de lectura/escritura en los archivos pasados como argumento.
     * @throws java.security.NoSuchProviderException El proveedor de datos referenciado no se encuentra disponible. Por defecto se emplea SunJCE.
     * @throws java.security.NoSuchAlgorithmException El proveedor de datos no soporta el algoritmo de cifrado.
     * @param bloqueArchivo Representa el bloque de datos a cifrar.
     * @param bloqueArchivo_cifrado Representa el bloque de datos cifrado.
     */
    public boolean CipherBlock(FileInputStream bloqueArchivo, FileOutputStream bloqueArchivo_cifrado, SecretKey clave) throws NoSuchAlgorithmException, IOException, InvalidKeyException, NoSuchPaddingException
    {
        //Si  Cube Cryptography Engine no esta activo salimos: 
        if (!GetActiveEngine()){WriteLog(Level.WARNING,"Bloque de datos NO cifrado. Cube Cryptography Engine no está activo.\n"); return false;}  
        
        Cipher codificador;      
        CipherInputStream cis;
        byte [] buffer_datos = new byte [long_buffer_data];
        int resultadoLectura = 0;
        
        try {
            codificador=Cipher.getInstance(algorithm_name);
            try {
                codificador.init(Cipher.ENCRYPT_MODE, clave);
            } catch (InvalidKeyException ex) {
                WriteLog(Level.SEVERE,"No se puede iniciar el cifrado de bloque con la clave pasada como argumento.\n");
                throw new InvalidKeyException();
            }
            cis = new CipherInputStream(bloqueArchivo, codificador);
            
            resultadoLectura = cis.read(buffer_datos);
            
           while (resultadoLectura != -1) {
                bloqueArchivo_cifrado.write(buffer_datos, 0, resultadoLectura);
                resultadoLectura = cis.read(buffer_datos);
                }
           
                bloqueArchivo_cifrado.close();
                cis.close();
           WriteLog(Level.FINE,"Bloque de datos cifrado.\n");     
           return true;
            
        } catch (NoSuchAlgorithmException ex) { 
            SetActiveEngine(false);
            WriteLog(Level.SEVERE,"El proveedor de cifrado no proporciona el algoritmo " + algorithm_name + ".\n");
           
            throw new NoSuchAlgorithmException(algorithm_name);
        } catch (NoSuchPaddingException ex) {
            WriteLog(Level.WARNING,"Esquema padding para " + algorithm_name+" invalido o no se encuentra disponible en el proveedor actual.\n");
            throw new NoSuchPaddingException();
        }
        
        catch (IOException ex) {
                WriteLog(Level.WARNING,"Error de I/O en el Block File de un archivo.\n");
                throw new IOException();
            }
       
    }
    
    
    /**
     * Descifra un bloque de datos almacenado en una archivo. Es usado para desencriptar los 
     * fragmentos de archivo.
     * @return TRUE: Bloque de datos descifrado con exito.<BR>
     * FALSE: El bloque de datos no se ha descifrado. Causa: Cube Cryptography Engine no esta activo.
     * @throws java.io.IOException Error de lectura/escritura en los archivos pasados como argumento.
     * @throws java.security.NoSuchProviderException El proveedor de datos referenciado no se encuentra disponible. Por defecto se emplea SunJCE.
     * @throws java.security.NoSuchAlgorithmException El proveedor de datos no soporta el algoritmo de cifrado.
     * @param bloqueArchivo Representa el bloque de datos descifrado.
     * @param bloqueArchivo_cifrado Representa el bloque de datos cifrado.
     */
    public boolean UnCipherBlock(FileInputStream bloqueArchivo_cifrado, FileOutputStream bloqueArchivo, SecretKey clave) throws NoSuchAlgorithmException, IOException, InvalidKeyException, NoSuchPaddingException
    {
        //Si  Cube Cryptography Engine no esta activo salimos: 
        if (!GetActiveEngine()){WriteLog(Level.WARNING,"Bloque de datos NO descifrado. Cube Cryptography Engine no está activo.\n"); return false;}  
        
        Cipher codificador;      
        CipherOutputStream  cos;
        byte [] buffer_datos = new byte [long_buffer_data];
        int resultadoLectura = 0;
        
        try {
            codificador=Cipher.getInstance(algorithm_name);
             try {
                codificador.init(Cipher.DECRYPT_MODE, clave);
            } catch (InvalidKeyException ex) {
                WriteLog(Level.SEVERE,"No se puede iniciar el cifrado de bloque con la clave pasada como argumento.\n");
                throw new InvalidKeyException();
            }
            cos = new CipherOutputStream (bloqueArchivo, codificador);
            
            resultadoLectura = bloqueArchivo_cifrado.read(buffer_datos);
            while(resultadoLectura != -1)
            {
                cos.write(buffer_datos, 0, resultadoLectura);
                resultadoLectura = bloqueArchivo_cifrado.read(buffer_datos);
                 
            }
            
            cos.flush();
            bloqueArchivo_cifrado.close();
            cos.close();
            WriteLog(Level.FINE,"Bloque de datos descifrado\n");
            
            return true;
            
        } catch (NoSuchAlgorithmException ex) { 
            SetActiveEngine(false);
            WriteLog(Level.SEVERE,"El proveedor de cifrado no proporciona el algoritmo " + algorithm_name + ".\n");
            
            throw new NoSuchAlgorithmException(algorithm_name);
        } catch (NoSuchPaddingException ex) {
            WriteLog(Level.WARNING,"Esquema padding para " + algorithm_name+" invalido o no se encuentra disponible en el proveedor actual.\n");
            throw new NoSuchPaddingException();
        }
        
        catch (IOException ex) {
                WriteLog(Level.WARNING,"Error de I/O en el Block File de un archivo.\n");
                throw new IOException();
            }
    }
    
    /**
     * Cifra un conjunto de bytes con algoritmo simetrico.
     * @return <B>true</B> datos cifrados. <B>false</B> Ocurrio un problema. Consultar el log.
     * @throws java.security.InvalidKeyException La clave proporcioada no es correcta.
     * @throws java.security.NoSuchAlgorithmException Algoritmo de cifrado No proporcionado por el conjunto de proveedores instalados en
     * Java.
     * @param clave Clave de cifrado.
     * @param data Datos origen.
     * @param destiny Resultado de la operacion.
     */
    public boolean CipherBytes(SecretKey clave, byte data[], byte [] destiny) throws InvalidKeyException, NoSuchAlgorithmException
    {
        //Si  Cube Cryptography Engine no esta activo salimos: 
        if (!GetActiveEngine()){WriteLog(Level.WARNING,"Bloque de datos NO descifrado. Cube Cryptography Engine no está activo.\n"); return false;}  
        
        Cipher codificador = null;      
            
        try {
            codificador=Cipher.getInstance(algorithm_name);
            codificador.init(Cipher.ENCRYPT_MODE, clave);
        } catch (InvalidKeyException ex) {
            WriteLog(Level.SEVERE,"No se puede iniciar el cifrado de bloque con la clave pasada como argumento.\n");
            throw new InvalidKeyException();
        } catch (NoSuchAlgorithmException ex) {
            WriteLog(Level.SEVERE,"El proveedor de cifrado no proporciona el algoritmo " + algorithm_name + ".\n");
            throw new NoSuchAlgorithmException();
        } catch (NoSuchPaddingException ex) {
            WriteLog(Level.WARNING,"Esquema padding para " +algorithm_name+" invalido o no se encuentra disponible en el proveedor actual.\n");
        }
        try {
            
            destiny = codificador.doFinal(data);
            return true;
        } catch (BadPaddingException ex) {
        } catch (IllegalBlockSizeException ex) {
            WriteLog(Level.WARNING,"El tamaño del buffer no es multiplo del bloque. Es un problema de padding.\n");
        }
        return false;
        
    }
    
    /**
     * descifra un conjunto de bytes con algoritmo simetrico.
     * @return <B>true</B> datos cifrados. <B>false</B> Ocurrio un problema. Consultar el log.
     * @throws java.security.InvalidKeyException La clave proporcioada no es correcta.
     * @throws java.security.NoSuchAlgorithmException Algoritmo de cifrado No proporcionado por el conjunto de proveedores instalados en
     * Java.
     * @param clave Clave de cifrado.
     * @param data Datos origen.
     * @param destiny Resultado de la operacion.
     */
    public boolean UnCipherBytes(SecretKey clave, byte data[], byte [] destiny) throws InvalidKeyException, NoSuchAlgorithmException
    {
        //Si  Cube Cryptography Engine no esta activo salimos: 
        if (!GetActiveEngine()){WriteLog(Level.WARNING,"Bloque de datos NO descifrado. Cube Cryptography Engine no está activo.\n"); return false;}  
        
        Cipher codificador = null;      
            
        try {
            codificador=Cipher.getInstance(algorithm_name);
            codificador.init(Cipher.DECRYPT_MODE, clave);
        } catch (InvalidKeyException ex) {
            WriteLog(Level.SEVERE,"No se puede iniciar el cifrado de bloque con la clave pasada como argumento.\n");
            throw new InvalidKeyException();
        } catch (NoSuchAlgorithmException ex) {
            WriteLog(Level.SEVERE,"El proveedor de cifrado no proporciona el algoritmo " + algorithm_name + ".\n");
            throw new NoSuchAlgorithmException();
        } catch (NoSuchPaddingException ex) {
            WriteLog(Level.WARNING,"Esquema padding para " +algorithm_name+" invalido o no se encuentra disponible en el proveedor actual.\n");
        }
        try {
            
            destiny = codificador.doFinal(data);
            return true;
        } catch (BadPaddingException ex) {
            WriteLog(Level.WARNING,"Ha ocurrido un error con el padding del conjuto de datos.\n");
        } catch (IllegalBlockSizeException ex) {}
        return false;
        
    }
    
    /**
     * Obtiene un resument del contenido aplicando SHA-1. El resultado es de 160 bits.
     * @param data Conjunto de datos al que aplicar el algoritmo resumen.
     * @return 160 bits que identifican de forma unica el contenido pasado como argumento. <B>null si Cube Cryptography Engine no esta activo.</B>
     */
    synchronized public byte [] Resumen(byte[] data)
    {
        if (!GetActiveEngine()){WriteLog(Level.WARNING,"RESUMEN HASH: No se puede obtener el resumen del contenido. Cube Cryptography Engine no está activo.\n"); return null;}
        return digest.digest(data);
    }
       
    
    /**
     * Metodo que permite consultar el estado del engine por varias hebras a la vez.
     * Para llevar a cabo tal accion se declara sincronizado y es la JVM la encargada
     * de usar el cerrojo par permitir la exclusión mutua de la clase,
     * ya que se trata de un metodo estático.
     * @return Determina si en Cube Cryptography Engine esta activo.
     */
    synchronized static public boolean GetActiveEngine(){return activeEngine;}
    /**
     * Metodo que permite establecer el estado del Cube Cryptogrphy engine por varias hebras a la vez.
     * Para llevar a cabo tal accion se declara sincronizado y es la JVM la encargada
     * de usar el cerrojo par permitir la exclusión mutua de la clase,
     * ya que se trata de un metodo estático.
     * @param value Valor true/false par determinar el estado del Cube Cryptogrphy engine
     */
    synchronized static private void SetActiveEngine(boolean value)
    {
        activeEngine=value;
        if (value) WriteLog(Level.SEVERE,"Iniciado: Cube Cryptography Engine.\n"); 
        else  WriteLog(Level.SEVERE,"Detenido: Cube Cryptography Engine.\n");
    }
    /**************************************************************************************************************/
    
    
    /**
     * Facilita la escritura de mensajes en el Logger solo si este está activo.
     * @see 
     */
    private static void WriteLog(Level nivel,String mensaje) {if (LogZero.Active()){mylog.log(nivel,mensaje);}}
    
}
