/*
 * Updater.java
 *
 * Created on 17 de octubre de 2007, 2:07
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package cube.common.updater;

import cube.common.postman.Postman;
import java.util.HashMap;
import java.util.Vector;
import org.apache.commons.io.FileUtils;
import org.xmlpull.v1.*;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import java.net.URL;
import java.net.URLConnection;

/**
 * Sistema MUY SIMPLE de actualizacion de paquetes. El paquete consisten en un unico
 * archivo Jar
 * @author victor Puertas
 */
public class Updater {
    
    /**
     * Direccion Http del servidor de actualizaciones.
     */
    private String updateAddress ="";
    /**
     * Prefijo para los archivos temporales generados por el updater.
     */
    static public final String tempFilesPrefix ="updater";
    
    //Clase File que representa al fichero temporal con la informacion de actualizacion:
    private File tempFile = null;
    
    //Vector que almacena un conjunto de estructuras tipo PackageData:
    private HashMap<String,PackageData> packages = new HashMap();
    
    /** Creates a new instance of Updater */
    public Updater() 
    {
    }
    
    /**
     * Descarga un archivo mediante protocolo Http.
     * @param address Direccion Http.
     * @param localFileName Ruta al archivo destino.
     * @return False si ocurrio algun problema.
     */
    public static void downloadFileByHttp(String address, File localFileName) throws IOException
    { 
        FileUtils.copyURLToFile(new URL(address),localFileName);
        
        
             /* OutputStream out = null;
		URLConnection conn = null;
		InputStream  in = null;
		try {
			URL url = new URL(address);
			out = new BufferedOutputStream( new FileOutputStream(localFileName) );
			conn = url.openConnection();
			in = conn.getInputStream();
			byte[] buffer = new byte[1024];
			int numRead;
			long numWritten = 0;
			while ((numRead = in.read(buffer)) != -1) {
				out.write(buffer, 0, numRead);
				numWritten += numRead;
			}
			System.out.println(localFileName + "\t" + numWritten);
		} catch (Exception exception) {
			
		} 
                finally 
                {
			try {
				if (in != null) { in.close(); }
				if (out != null) { out.close(); }
			} catch (IOException ioe) {}
		}
                
              */
	}
    public void getUpdate(String address) throws IOException
    {
        tempFile = File.createTempFile(tempFilesPrefix, "."+Postman.extensionTempFile, new File(Postman.getCubeDirectoryRoot()+File.separator+Postman.directoryTemporal));
        downloadFileByHttp(address,tempFile);
        
             parserUpdateFile(tempFile);
        
        
        //Lo marcamos para borrar:
        tempFile.deleteOnExit();
    }
    
    /**
     * Obtiene un vector con la informacion de los paquetes.
     */
    public HashMap<String,PackageData>  getPackageData() { return packages; }
    /**
     * Parser del XML de actualizacion. Se encarga de interpretar dicho archivo y generar las 
     * estructuras de inforamcion adecuadas. 
     */
    private void parserUpdateFile(File file)
    {
        FileInputStream fin = null;
        try { fin = new FileInputStream(file); } catch (FileNotFoundException ex) { return; }
       
        PackageData paqueteAux = null;
        
        try
        {
            XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
            factory.setNamespaceAware(true);
            XmlPullParser xpp = factory.newPullParser();
            xpp.setInput(fin,null);
            
            //Almacena el ultimo tag procesado:
            String lastTag = "";
                      
            //Gestion de los eventos
            int eventType = xpp.getEventType();
            while (eventType != XmlPullParser.END_DOCUMENT) 
             {
                switch(eventType)
                {
                    case XmlPullParser.START_TAG:
                        lastTag = xpp.getName();
                          
                        if(xpp.getName()=="Package")
                        {
                            paqueteAux = new PackageData();
                        }
                        break;
                        
                    case XmlPullParser.END_TAG:
                        lastTag = "";
                        
                        if(xpp.getName()=="Package")
                        {
                            //Compribamos el tamaño de la fila para ver si hay algun error:
                            //if(fila.size()==NUMBER_DATA)
                            packages.put(paqueteAux.internal_name,paqueteAux);
                            paqueteAux = null;
                        }
                        break;
                        
                    case XmlPullParser.TEXT:
                        if(lastTag.equals("")) break;
                        
                        if(lastTag.equals("name") && paqueteAux != null)
                            paqueteAux.internal_name = xpp.getText();
                   
                        if(lastTag.equals("author") && paqueteAux != null)
                            paqueteAux.author = xpp.getText();
                        
                        if(lastTag.equals("copyright") && paqueteAux != null)
                           paqueteAux.copyright = xpp.getText();
                        
                        if(lastTag.equals("description") && paqueteAux != null)
                           paqueteAux.description = xpp.getText();
                        
                        if(lastTag.equals("history") && paqueteAux != null)
                           paqueteAux.history = xpp.getText();
                        
                        if(lastTag.equals("size") && paqueteAux != null)
                            try{ paqueteAux.size = Long.parseLong(xpp.getText()); }
                            catch (NumberFormatException err){}
                        
                        if(lastTag.equals("version_main") && paqueteAux != null)
                            try{ paqueteAux.version_main = Integer.parseInt(xpp.getText()); }
                            catch (NumberFormatException err){}
                        
                        if(lastTag.equals("version_minor") && paqueteAux != null)
                            try{ paqueteAux.version_second = Integer.parseInt(xpp.getText()); }
                            catch (NumberFormatException err){}
                        
                        if(lastTag.equals("version_rev") && paqueteAux != null)
                            try{ paqueteAux.version_rev = Integer.parseInt(xpp.getText()); }
                            catch (NumberFormatException err){}
                        
                        if(lastTag.equals("Package_name") && paqueteAux != null)
                             paqueteAux.dependences.add(xpp.getText());
                        
                         if(lastTag.equals("mirror") && paqueteAux != null)
                             paqueteAux.mirror.add(xpp.getText());
                            
                        
                        break;
                }
                eventType = xpp.next();
            }
         
        }
        catch (XmlPullParserException ex) { return; }
        catch (IOException ex) { return; }
    }
    
}
