/*
* Title			:   util.java
* Package		:   cube.common.util
* Project		:   Cube Platform
* version               :   Alfa 0
* Description           :   Conjunto de utilidades empleadas en Cube.
*                           		
* Created on            :   02/07/2007 
* Author		:   Victor Puertas Garcia.
* Copyright		:   (c) 2007, Victor Puertas Garcia. vpgugr@gmail.com 
* 					   
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later  version.
* See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
* 
* HISTORIAL DE CAMBIOS. Formato: <fecha> <autor> <descripcion>
*
*
*/

package cube.common.util;

import cube.common.updater.PackageData;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URLConnection;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import org.apache.commons.io.FilenameUtils;
import org.xmlpull.v1.*;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.net.URL;

/**
 * Contiene utilidades de diverso tipo empleado en las clases.
 * @author victor Puertas
 */
public class util {
    /**
     * Direccion del script php que permite obtener la direccion IP externa.
     */
    static private String URL_EXTERNAL_IO_SCRIPT = "http://cube-grid.sourceforge.net/support/external_ip.php";
    
    /**
     * Obtiene una marca de tiempo tipo Unix: Numero de milisegundos trascurridos desde la media noche
     * 1 de junio de 1970.
     */
    static public long getUnixTime() 
    {
        java.util.Date date = new java.util.Date();
        return date.getTime(); 
    }
    
    //public util() {}
    
    /**
     * Concatena un array de bytes a otro.
     * @param org Array de bytes origen de los datos.
     * @param dest Array de bytes en donde se concatenaran los bytes.
     * @param start Elemento desde el que se empezara la operacion en el array destino.
     * @param size Numero de bytes que se copiaran desde el origen.
     * @return Devuelve el indice por el que seguir insertando en el array destino.
     */
     static public int concatbytes(byte org [], byte dest [], int start, int size)
    {
        int indice=start;
        
        for(int i = 0; i<size; i++)
        {
            dest[indice] = org[i];
            indice++;
        }
        return indice;
    }
     
  
	/**
	 * Constantes Hex:
	 */
	private static final char[] hexArray =
		{
			'0',
			'1',
			'2',
			'3',
			'4',
			'5',
			'6',
			'7',
			'8',
			'9',
			'a',
			'b',
			'c',
			'd',
			'e',
			'f' };

	/**
	 * A lookup table for the high order hex character.
	 */
	private static final byte[] high = new byte[16];

	/**
	 * A lookup table for the low order hex character.
	 */
	private static final byte[] low = new byte[16];

	static {
		byte b;

		for (int i = 0; i < low.length; i++)
		{
			b = (byte) i;
			low[i] = (byte) (b & 0x0f);
		}

		for (int i = 0; i < high.length; i++)
		{
			b = (byte) i;
			high[i] = (byte) ((b << 4) & 0xf0);
		}
	}

	/**
	 * Converts a byte[] array into a Hex string
	 * @param inByteArray
	 * @return string
	 */
	public static String bytesToHexString(byte[] inByteArray)
	{
		return bytesToHexString(inByteArray, 0, inByteArray.length);
	}

	/**
	 * Converts the given byte array into a hex string representation.
	 * @param inByteArray
	 * @param offset
	 * @param len
	 * 
	 * @return  a hex String representation of the byte array.
	 */
	public static String bytesToHexString(
		byte[] inByteArray,
		int offset,
		int len)
	{
		if (inByteArray == null)
			return null;

		int position;
		StringBuffer returnBuffer = new StringBuffer();

		for (position = offset; position < len; position++)
		{
			returnBuffer.append(
				hexArray[((inByteArray[position] >> 4) & 0x0f)]);
			returnBuffer.append(hexArray[(inByteArray[position] & 0x0f)]);
		}

		return returnBuffer.toString();
	}

	/**
	 * This function accepts a number of bytes and returns a long
	 * integer.  Note that if the input is 8 bytes, and the high bit of the
	 * 0 byte is set, this will come out negative. 
	 *
	 * @param  bytes the byte array to convert to a long integer representation.
	 * @return a long integer representation of the byte array.
	 */
	public static long bytesToLong(byte[] bytes)
	{
		if (bytes.length > 8)
			throw new IllegalArgumentException("Deben ser 4 bytes o menos");

		long returnLong = 0;

		for (int n = 0; n < bytes.length; n++)
		{
			returnLong <<= 8;

			long aByte = bytes[n] < 0 ? bytes[n] + 256 : bytes[n];

			returnLong = returnLong | aByte;
		}

		return returnLong;
	}

	/**
	 * This function accepts a number of bytes and returns an
	 * integer.  Note that if the input is 4 bytes, and the high bit of the
	 * 0 byte is set, this will come out negative. 
	 *
	 * @param  bytes the byte array to convert to a long integer representation.
	 * @return a long integer representation of the byte array.
	 */
	public static int bytesToInt(byte[] bytes)
	{
		if (bytes.length > 4)
			throw new IllegalArgumentException("Deben ser 4 bytes o menos");

		int returnInt = 0;

		for (int n = 0; n < bytes.length; n++)
		{
			returnInt <<= 8;

			int aByte = bytes[n] < 0 ? bytes[n] + 256 : bytes[n];

			returnInt = returnInt | aByte;
		}

		return returnInt;
	}

	/**
	 * Returns the index for the given hex character in the byte array lookup array.
	 * This is used for both the hig order and low order hex characters.
	 *
	 * @param   c the character to get the lookup index.
	 * @return  an index into a byte array lookup table for the given hex character.
	 */
	private static int getIndex(char c)
	{
		if (('0' <= c) && (c <= '9'))
		{
			return ((byte) c - (byte) '0');
		}
		else if (('a' <= c) && (c <= 'f'))
		{
			return ((byte) c - (byte) 'a' + 10);
		}
		else if (('A' <= c) && (c <= 'F'))
		{
			return ((byte) c - (byte) 'A' + 10);
		}
		else
		{
			return -1;
		}
	}

	/**
	 * This method accepts a hex string and returns a byte array. The string must 
	 * represent an integer number of bytes.
	 *
	 * @param   str - the hex string to convert to byte array representation.
	 * @return  the byte array representation of the hex string.
	 */
	public static byte[] hexStringToBytes(String str)
	{
		byte b;
		byte b2;
		int len = str.length();
		byte[] retval = new byte[len / 2];

		int j = 0;

		for (int i = 0; i < len; i += 2)
		{
			b = high[getIndex(str.charAt(i))];
			b2 = low[getIndex(str.charAt(i + 1))];
			retval[j++] = (byte) (b | b2);
		}

		return retval;
	}

	/**
	 * Converts a integer to a byte array representation.
	 *
	 * @param   i the integer to convert to byte array representation.
	 * @return  the byte array representation of the integer.
	 */
	public static byte[] intToBytes(int i)
	{
		int ii = i;

		byte[] returnBytes = new byte[8];

		for (int n = 3; n >= 0; n--)
		{
			returnBytes[n] = (byte) ii;
			ii = ii >>> 8;
		}

		return returnBytes;
	}

	/**
	 * Converts a long integer to a byte array representation.  If the target
	 * array is shorter than needed, the hi bytes of the integer will be
	 * truncated.
	 *
	 * @param l the long integer to convert to byte array representation.
	 * @param len the size of the byte array to return
	 * @return  the byte array representation of the long integer.
	 */
	public static byte[] longToBytes(long l, int len)
	{
		byte[] returnValue = new byte[len];
		longToBytes(l, returnValue, 0, len);
		return returnValue;
	}

	/**
	 * Converts a long integer to a byte array representation.  If the target
	 * array is shorter than needed, the hi bytes of the integer will be
	 * truncated.
	 *
	 * @param l the long integer to convert to byte array representation.
	 * @param b
	 * @param offset
	 * @param len the size of the byte array to return
	 */
	public static void longToBytes(long l, byte[] b, int offset, int len)
	{
		int startPoint = len < 8 ? len - 1 : 7;

		if (startPoint < 7)
			l = l & (0xFFFFFFFFFFFFFFFFL >> (8 * (7 - startPoint)));

		for (int n = len - 1; n >= 0; n--)
		{
			b[offset + n] = (byte) l;
			l = l >> 8;
		}
	}

	/**
	 * Converts a byte in range -128 <= i <= 127 to an int in 
	 * range 0 <= i <= 255.
	 *
	 * @param i the byte to convert
	 * @return the converted integer
	 */
	public static int unsignedByteToInt(byte i)
	{
		return i < 0 ? (i + 256) : i;
	}
	

	/**
	 * Use if you don't want to have to catch a million
	 * UnsupportedEncodingExceptions.
	 * @param in
	 * @param encoding
	 * @return byte[]
	 */
	public static byte[] stringToByteArray(String in, String encoding)
	{
		try
		{
			return in.getBytes(encoding);
		}
		catch (UnsupportedEncodingException e)
		{
			throw new RuntimeException(
				"Codificación no soportada: " + encoding);
		}
	}

	/**
	 * Use if you don't want to have to catch a million
	 * UnsupportedEncodingExceptions.
	 * @param in
	 * @param encoding
	 * @return String
	 */
	public static String byteArrayToString(byte[] in, String encoding)
	{
		try
		{
			return new String(in, encoding);
		}
		catch (UnsupportedEncodingException e)
		{
			throw new RuntimeException(
				"Codificación no soportada: " + encoding);
		}
	}

	/**
	 * Convierte de BigInteger a byte array.
	 * 
	 * @return  El BigInteger como byte array
	 */
	public static byte[] bigIntegerToUnsigedBytes(BigInteger b)
	{
		// Conversion the BigInteger to a byte array
		byte[] bytes = b.toByteArray();

		if (b.signum() == -1)
		{
			throw new IllegalArgumentException("Solo BigIntegers");
		}

		while (bytes[0] == 0x00)
		{
			byte[] bytes2 = new byte[bytes.length - 1];
			System.arraycopy(bytes, 1, bytes2, 0, bytes2.length);
			bytes = bytes2;
		}

		return bytes;
	}   
        
        /**
         * Serializa un objeto en el archivo indicado.
         */
        static public void serializeObject(java.io.FileOutputStream fout, Object obj) throws IOException
        {
            java.io.ObjectOutputStream out = null;
            out = new java.io.ObjectOutputStream(fout); 
            out.writeObject(obj);
            out.close();
        }
        
        /**
         * Deserializa un objeto del archivo indicado.
         */
        static public Object unSerializeObject(java.io.FileInputStream fin) throws IOException, ClassNotFoundException
        {
            java.io.ObjectInputStream in = null;
            in = new java.io.ObjectInputStream(fin); 
            Object aux = in.readObject();
            in.close();
            return aux;
        }
        
        /**
         * Lee un fichero con informacion de paquetes.
         * @return Informacion del paquete. <b>Null</b> si error.
         */
        static public PackageData readPackageInfo(Object obj, String resource)
        {
            PackageData paqueteAux = null;
            URL recurso = obj.getClass().getResource(resource);
            
            //PARSER XML:
            try
            {
                    XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
                    factory.setNamespaceAware(true);
                    XmlPullParser xpp = factory.newPullParser();
                    xpp.setInput(recurso.openStream(),null);

                    //Almacena el ultimo tag procesado:
                    String lastTag = "";

                    //Gestion de los eventos
                    int eventType = xpp.getEventType();
                    while (eventType != XmlPullParser.END_DOCUMENT) 
                     {
                        switch(eventType)
                        {
                            case XmlPullParser.START_TAG:
                                lastTag = xpp.getName();

                                if(xpp.getName()=="package")
                                {
                                    paqueteAux = new PackageData();
                                }
                                break;

                            case XmlPullParser.END_TAG:
                                lastTag = "";
                                if(xpp.getName()=="package") { return paqueteAux; }
                                break;

                            case XmlPullParser.TEXT:
                                if(lastTag.equals("")) break;

                                if(lastTag.equals("internal_name") && paqueteAux != null)
                                    paqueteAux.internal_name = xpp.getText();
                                
                                if(lastTag.equals("public_name") && paqueteAux != null)
                                    paqueteAux.public_name = xpp.getText();

                                if(lastTag.equals("author") && paqueteAux != null)
                                    paqueteAux.author = xpp.getText();

                                if(lastTag.equals("copyright") && paqueteAux != null)
                                   paqueteAux.copyright = xpp.getText();

                                if(lastTag.equals("description") && paqueteAux != null)
                                   paqueteAux.description = xpp.getText();

                                if(lastTag.equals("version_main") && paqueteAux != null)
                                    try{ paqueteAux.version_main = Integer.parseInt(xpp.getText()); }
                                    catch (NumberFormatException err){}

                                if(lastTag.equals("version_second") && paqueteAux != null)
                                    try{ paqueteAux.version_second = Integer.parseInt(xpp.getText()); }
                                    catch (NumberFormatException err){}

                                if(lastTag.equals("version_rev") && paqueteAux != null)
                                    try{ paqueteAux.version_rev = Integer.parseInt(xpp.getText()); }
                                    catch (NumberFormatException err){}
                                
                                 if(lastTag.equals("codename") && paqueteAux != null)
                                   paqueteAux.codename = xpp.getText();
                                break;
                        }
                        eventType = xpp.next();
                    }
         
        }
        catch (XmlPullParserException ex) {}
        catch (IOException ex) {}
        catch (NullPointerException ex){}
        return null;
            
     }
     
    /**
     * Descomprime un fichero ZIP en el destino especificado.
     */
     static public void descompressZip(File zip, File destination) throws IOException
     {
         int BUFFER_SIZE = 8192; //Tamaño de buffer: 8.2 Kbytes.
                // Create a ZipInputStream to read the zip file
                BufferedOutputStream dest = null;
                FileInputStream fis = new FileInputStream( zip );
                ZipInputStream zis = new ZipInputStream( new BufferedInputStream( fis ) );
                // Loop over all of the entries in the zip file
                int count;
                
                byte data[] = new byte[ BUFFER_SIZE ];
                ZipEntry entry;
                while( ( entry = zis.getNextEntry() ) != null )
                {
                    if( !entry.isDirectory() )
                    {
                        String entryName = entry.getName();
                        File subDirectorio = new File(destination,File.separator+entryName);
                        
                        //Creamos el directorio destino:
                        new File(FilenameUtils.getFullPath(subDirectorio.getAbsolutePath())).mkdir();
                        
                        //prepareFileDirectories( destination, entryName );
                        String destFN = destination + File.separator + entry.getName();
                        System.err.println(destFN);
                        // Write the file to the file system
                        FileOutputStream fos = new FileOutputStream( destFN );
                        dest = new BufferedOutputStream( fos, BUFFER_SIZE );
                        while( (count = zis.read( data, 0, BUFFER_SIZE ) ) != -1 )
                        {
                            dest.write( data, 0, count );
                        }
                        dest.flush();
                        dest.close();
                    }
                }
                
                zis.close();
    }
     
     /**
      * Convierte una cantidad expresada en bytes a la unidad mas proxima.
      * ejemplo: 1024 bytes -> 1 KiloByte. Soporta hasta 1024 Terabytes. Mas
      * alla de esta cantidad devolvera resultados en Terabytes.
      */
     static public String parserBytes2Size(long value)
     {
         if(value>1099511627776L)
           return new String(Long.toString(value/1099511627776L)+"."+Long.toString(value%1099511627776L).substring(0,1)+" Terabytes");
        else
            if(value>=1073741824L)
                return new String(Long.toString(value/1073741824L)+"."+Long.toString(value%1073741824L).substring(0,1)+" Gigabytes");
            else
                if(value>=1048576)
                    return new String(Long.toString(value/1048576)+"."+Long.toString(value%1048576).substring(0,1)+ " Megabytes");
                else
                    if(value>=1024)
                        return new String(Long.toString(value/1024)+" Kilobytes");
                    else
                        return new String(Long.toString(value)+" Bytes");
     }
     
    /**
     * Obtiene la IP con la que una maquina sale a Internet
     * haciendo uso de un script PHP situado en un servidor.
     * @return Ip externa. <b>null</b> si ha ocurrido un error.
     * @todo Terminar de implementar esta funcion.
     */ 
    static public String netExternalIPAddress()
    {
       
                    try
                    {

                      URL url = new URL(URL_EXTERNAL_IO_SCRIPT);

                      URLConnection connection = url.openConnection();
                      connection.setReadTimeout(3000);
                      connection.setDoInput(true);
                      InputStream inStream = connection.getInputStream();

                      try
                        {
                                XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
                                factory.setNamespaceAware(true);
                                XmlPullParser xpp = factory.newPullParser();
                                xpp.setInput(inStream, null);

                                //Almacena el ultimo tag procesado:
                                String lastTag = "";

                                //Gestion de los eventos
                                int eventType = xpp.getEventType();
                                while (eventType != XmlPullParser.END_DOCUMENT) 
                                 {
                                    switch(eventType)
                                    {
                                        case XmlPullParser.START_TAG:
                                            lastTag = xpp.getName();       
                                            break;

                                        case XmlPullParser.TEXT:
                                            if(lastTag.equals("external_IP"))
                                            {
                                               Pattern p = Pattern.compile("\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}");
                                               if(p.matcher(xpp.getText()).matches())
                                                    return xpp.getText();

                                               return null;
                                            }
                                    }
                                    eventType = xpp.next();
                                }
                        }
                      catch (XmlPullParserException ex) {System.out.println("Se ha producido un error en el parser XML al obtener la IP externa.");}       
                    }
                    catch (Exception e) { System.out.println("No se puede obtener la IP externa del equipo."); }
                    return null;
        
        //El resultado de ejecutar el script es un documento XML que contiene
        //un elemento: <external_IP>IP externa</external_IP>
       
   }
    
}